# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
# from past.builtins import basestring

from . import amplpython


class Runnable(amplpython.Runnable):
    """
    Represent an object with only one function, which is called as a callback
    after an async execution has finished. Inherit from this class and use an
    instance in
    :func:`~amplpy.AMPL.eval_async` / :func:`~amplpy.AMPL.evalAsync`,
    :func:`~amplpy.AMPL.read_async` / :func:`~amplpy.AMPL.readAsync`,
    :func:`~amplpy.AMPL.read_data_async` / :func:`~amplpy.AMPL.readDataAsync`,
    :func:`~amplpy.AMPL.solve_async` / :func:`~amplpy.AMPL.solveAsync`.
    """

    def run(self):
        """
        Function called when the execution of the async operation is finished.
        """



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
# from past.builtins import basestring

from .iterators import EnvIterator
from .base import BaseClass
from . import amplpython


class Environment(BaseClass):
    """
    This class provides access to the environment variables and provides
    facilities to specify where to load the underlying AMPL interpreter.
    """

    def __init__(self, binary_directory=None, binary_name=None):
        """
        Constructor with ability to select the location of the AMPL binary.
        Note that if binaryDirectory is set, the automatic lookup for an AMPL
        executable will not be executed.

        Args:
            binary_directory: The directory in which look for the AMPL executable.
            binary_name: The name of the AMPL executable.
        """
        if binary_directory is None:
            binary_directory = ""
        if binary_name is None:
            binary_name = ""
        super(Environment, self).__init__(
            amplpython.Environment(binary_directory, binary_name))

    def __iter__(self):
        return EnvIterator(self._impl)

    def __setitem__(self, name, value):
        """
        Add an environment variable to the environment, or change its value if
        already defined.

        Args:
            name: Name of the environment variable.

            value: Value to be assigned.
        """
        self._impl.put(name, value)

    def __getitem__(self, name):
        """
        Searches the current object for an environment variable called name and
        returns an iterator to it if found, otherwise it returns `None`.
        """
        iterator = self._impl.find(name)
        if iterator == self._impl.end():
            return None
        else:
            return iterator.second()

    def set_bin_dir(self, binary_directory):
        """
        Set the location where AMPL API will search for the AMPL executable.

        Args:
            binary_directory: The directory in which look for the AMPL executable.
        """
        self._impl.setBinDir(binary_directory)

    def get_bin_dir(self):
        """
        Get the location where AMPL API will search for the AMPL executable.
        """
        return self._impl.getBinDir()

    def set_bin_name(self, binary_name):
        """
        Set the name of the AMPL executable.

        Args:
            binary_name: The name of the AMPL executable.
        """
        self._impl.setBinName(binary_name)

    def get_bin_name(self):
        """
        Get the name of the AMPL executable.
        """
        return self._impl.getBinName()

    # Aliases
    getBinDir = get_bin_dir
    getBinName = get_bin_name
    setBinDir = set_bin_dir
    setBinName = set_bin_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted

from . import TestBase
import unittest
import amplpy


class TestProperties(TestBase.TestBase):
    """Test Properties."""

    def testVarProp(self):
        ampl = self.ampl
        ampl.eval('var x;')
        ampl.var['x'] = 10
        self.assertEqual(
            ampl.var['x'].value(),
            ampl.getVariable('x').value()
        )
        self.assertEqual(
            len(list(ampl.var)),
            len(list(ampl.getVariables()))
        )

    def testConProp(self):
        ampl = self.ampl
        ampl.eval('var x; s.t. c: x = 10;')
        ampl.con['c'] = 10
        self.assertEqual(
            ampl.con['c'].dual(),
            ampl.getConstraint('c').dual()
        )
        self.assertEqual(
            len(list(ampl.con)),
            len(list(ampl.getConstraints()))
        )

    def testObjProp(self):
        ampl = self.ampl
        ampl.eval('var x; maximize obj: x;')
        self.assertEqual(
            ampl.obj['obj'].name(),
            ampl.getObjective('obj').name()
        )
        self.assertEqual(
            len(list(ampl.obj)),
            len(list(ampl.getObjectives()))
        )

    def testSetProp(self):
        ampl = self.ampl
        ampl.eval('set s;')
        ampl.set['s'] = [1, 2, 3]
        self.assertEqual(
            ampl.set['s'].name(),
            ampl.getSet('s').name()
        )
        self.assertEqual(ampl.set['s'].size(), 3)
        self.assertEqual(
            len(list(ampl.set)),
            len(list(ampl.getSets()))
        )

    def testParamProp(self):
        ampl = self.ampl
        ampl.eval('param p1; param p2{1..2};')
        self.assertEqual(
            ampl.param['p1'].name(),
            ampl.getParameter('p1').name()
        )
        ampl.param['p1'] = 3
        self.assertEqual(ampl.param['p1'].value(), 3)
        ampl.param['p2'] = {1: 10, 2: 20}
        self.assertEqual(ampl.param['p2'][1], 10)
        self.assertEqual(ampl.param['p2'][2], 20)
        self.assertEqual(
            len(list(ampl.param)),
            len(list(ampl.getParameters()))
        )

    def testOptionProp(self):
        ampl = self.ampl
        ampl.option['solver'] = 'gurobi'
        self.assertEqual(
            ampl.option['solver'],
            ampl.getOption('solver')
        )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted

import unittest
import tempfile
import shutil
import os
from .context import amplpy


# For MSYS2, MINGW, etc., run with:
# $ REAL_ROOT=`cygpath -w /` python -m amplpy.tests
REAL_ROOT = os.environ.get('REAL_ROOT', None)


class TestBase(unittest.TestCase):
    def setUp(self):
        print("Method:", self._testMethodName)
        self.ampl = amplpy.AMPL()
        self.dirpath = tempfile.mkdtemp()

    def _tmpfile(self, filename):
        return os.path.join(self.dirpath, filename)

    def _real_filename(self, filename):
        # Workaround for MSYS2, MINGW paths
        if REAL_ROOT is not None and filename.startswith('/'):
            filename = filename.replace('/', REAL_ROOT, 1)
        return filename

    def str2file(self, filename, content):
        fullpath = self._tmpfile(filename)
        with open(fullpath, 'w') as file:
            print(content, file=file)
        return self._real_filename(fullpath)

    def tmpfile(self, filename):
        return self._real_filename(self._tmpfile(filename))

    def tearDown(self):
        self.ampl.close()
        shutil.rmtree(self.dirpath)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
# from past.builtins import basestring

from .entity import Entity


class Constraint(Entity):
    """
    Bases: :class:`~amplpy.Entity`.

    This class represents an algebraic or logical constraint. In case the
    constraint is scalar, its values can be accessed via functions like
    :func:`~amplpy.Constraint.body` and :func:`~amplpy.Constraint.dual`.
    All the AMPL suffixes for constraints (see
    http://www.ampl.com/NEW/suffbuiltin.html)
    are available through methods of this class with the same name (and methods
    of ConstraintInstance for indexed constraints).

    Note that, since this class represents both algebraic and logical
    constraints, some suffixes might not be available for every entity.

    An TypeError is thrown if one of such methods is called for
    a non-scalar constraint and if a method corresponding to a suffix which is
    not supported by the type of the constraint is called.
    A runtime error is thrown if any property of an entity which has been
    deleted from the underlying interpreter is accessed.

    To gain access to all the values in an entity (for all instances and all
    suffixes for that entities), see
    :func:`~amplpy.Entity.get_values` / :func:`~amplpy.Entity.getValues`
    and the :class:`~amplpy.DataFrame` class.
    """

    def __init__(self, _impl):
        Entity.__init__(self, _impl, Constraint)

    def is_logical(self):
        """
        Check if the constraint is a logical constraint. Returns True if
        logical. The available suffixes differ between logical and non logical
        constraints. See http://www.ampl.com/NEW/suffbuiltin.html for a list of
        the available suffixes for algebraic constraints. The suffixes
        available for logical constraints are marked on the method description
        by "Valid only for logical constraints".
        """
        try:
            return self._impl.isLogical()
        except AttributeError:
            return False

    def drop(self):
        """
        Drop all instances in this constraint entity, corresponding to the AMPL
        code: `drop constraintname;`.
        """
        self._impl.drop()

    def restore(self):
        """
        Restore all instances in this constraint entity, corresponding to the
        AMPL code: `restore constraintname;`.
        """
        self._impl.restore()

    def body(self):
        """
        Get the current value of the constraint's body.
        """
        return self._impl.body()

    def astatus(self):
        """
        Get the current AMPL status (dropped, presolved, or substituted out).
        """
        return self._impl.astatus()

    def defvar(self):
        """
        Get the index in `_var` of "defined variable" substituted out by the
        constraint.
        """
        return self._impl.defvar()

    def dinit(self):
        """
        Get the current initial guess for the constraint's dual variable.
        """
        return self._impl.dinit()

    def dinit0(self):
        """
        Get the original initial guess for the constraint's dual variable.
        """
        return self._impl.dinit0()

    def dual(self):
        """
        Get the current value of the constraint's dual variable.

        Note that dual values are often reset by the underlying AMPL
        interpreter by the presolve functionalities triggered by some methods.
        A possible workaround is to set the option `presolve` to `false`
        (see :func:`~amplpy.AMPL.setOption`).
        """
        return self._impl.dual()

    def lb(self):
        """
        Get the current value of the constraint's lower bound.
        """
        return self._impl.lb()

    def ub(self):
        """
        Get the current value of the constraint's upper bound.
        """
        return self._impl.ub()

    def lbs(self):
        """
        Get the constraint lower bound sent to the solver (reflecting
        adjustment for fixed variables).
        """
        return self._impl.lbs()

    def ubs(self):
        """
        Get the constraint upper bound sent to the solver (reflecting
        adjustment for fixed variables).
        """
        return self._impl.ubs()

    def ldual(self):
        """
        Get the current dual value associated with the lower bound.
        """
        return self._impl.ldual()

    def udual(self):
        """
        Get the current dual value associated with the upper bounds.
        """
        return self._impl.udual()

    def lslack(self):
        """
        Get the slack at lower bound `body - lb`.
        """
        return self._impl.lslack()

    def uslack(self):
        """
        Get the slack at upper bound `ub - body`.
        """
        return self._impl.uslack()

    def slack(self):
        """
        Constraint slack (the lesser of lslack and uslack).
        """
        return self._impl.slack()

    def sstatus(self):
        """
        Get the solver status (basis status of constraint's slack or artificial
        variable).
        """
        return self._impl.sstatus()

    def status(self):
        """
        Get the AMPL status if not `in`, otherwise solver status.
        """
        return self._impl.status()

    def set_dual(self, dual):
        """
        Set the value of the dual variable associated to this constraint (valid
        only if the constraint is scalar). Equivalent to the AMPL statement:

        `let c := dual;`

        Note that dual values are often reset by the underlying AMPL
        interpreter by the presolve functionalities triggered by some methods.
        A possible workaround is to set the option `presolve` to `False`
        (see :func:`~amplpy.AMPL.setOption`).

        Args:
            dual: The value to be assigned to the dual variable.
        """
        self._impl.setDual(dual)

    def val(self):
        """
        Get the AMPL val suffix. Valid only for logical constraints.
        """
        if self.is_logical():
            return self._impl.val()
        else:
            return None

    # Aliases
    isLogical = is_logical
    setDual = set_dual



# -*- coding: utf-8 -*-
import os
import sys
import ctypes
import platform

if platform.system().startswith(('Windows', 'MSYS', 'CYGWIN', 'MINGW')):
    libbase = os.path.join(os.path.dirname(__file__), 'cppinterface', 'lib')
    lib32 = os.path.join(libbase, 'intel32')
    lib64 = os.path.join(libbase, 'amd64')
    from glob import glob
    try:
        if ctypes.sizeof(ctypes.c_voidp) == 4:
            dllfile = glob(lib32 + '/*.dll')[0]
        else:
            dllfile = glob(lib64 + '/*.dll')[0]
        ctypes.CDLL(dllfile)
    except Exception as exp:
        raise exp

sys.path.append(os.path.join(os.path.dirname(__file__), 'cppinterface'))
try:
    from amplpython import _READTABLE, _WRITETABLE
    from amplpython import *
except Exception as exp:
    raise exp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
import unittest
import os

# from builtins import map, range, object, zip, sorted
from past.builtins import basestring

import amplpy
from . import TestBase


class TestAMPL(TestBase.TestBase):
    """Test AMPL."""

    def test_ampl(self):
        from amplpy import Set, Parameter, Variable, Constraint, Objective
        ampl = self.ampl
        self.assertEqual(ampl.get_data('1..3').get_num_rows(), 3)
        self.assertEqual(ampl.get_data('1..3').get_num_cols(), 1)
        ampl.eval('set X := 1..10;')
        self.assertTrue(isinstance(ampl.get_entity('X'), amplpy.Entity))
        self.assertEqual(ampl.get_entity('X').get_values().get_num_rows(), 10)
        self.assertEqual(ampl.get_data('X').get_num_rows(), 10)

        with self.assertRaises(RuntimeError):
            self.assertRaises(ampl.get_data('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.get_entity('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.get_set('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.get_parameter('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.get_variable('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.get_constraint('XXX'))
        with self.assertRaises(TypeError):
            self.assertRaises(ampl.get_objective('XXX'))
        self.assertEqual(len(ampl.get_sets()), 1)
        ampl.reset()
        self.assertEqual(len(ampl.get_sets()), 0)
        ampl.eval('set _s; param _p; var _v; s.t. _c: _v = 0; maximize _o: 0;')
        self.assertTrue(isinstance(ampl.get_set('_s'), Set))
        self.assertTrue(isinstance(ampl.get_parameter('_p'), Parameter))
        self.assertTrue(isinstance(ampl.get_variable('_v'), Variable))
        self.assertTrue(isinstance(ampl.get_constraint('_c'), Constraint))
        self.assertTrue(isinstance(ampl.get_objective('_o'), Objective))
        self.assertEqual(len(ampl.get_sets()), 1)
        self.assertEqual(len(ampl.get_parameters()), 1)
        self.assertEqual(len(ampl.get_variables()), 1)
        self.assertEqual(len(ampl.get_constraints()), 1)
        self.assertEqual(len(ampl.get_objectives()), 1)
        ampl.reset()
        with self.assertRaises(ValueError):
            ampl.eval('X')
        self.assertTrue(ampl.is_running())
        self.assertFalse(ampl.is_busy())

    def test_getdata_multi(self):
        ampl = self.ampl
        ampl.eval('''
        param p1{i in 1..10} := 1*i;
        param p2{i in 1..10} := 2*i;
        param p3{i in 0..10: i >= 1} := 3*i;
        ''')
        df = ampl.get_data("p1", "p2", "p3")
        df = ampl.get_data("p1", "p2", "p3")
        self.assertEqual(list(df.get_column("p1")), [
                         i for i in range(1, 10+1)])
        self.assertEqual(list(df.get_column("p2")), [
                         2*i for i in range(1, 10+1)])
        self.assertEqual(list(df.get_column("p3")), [
                         3*i for i in range(1, 10+1)])
        self.assertEqual(df.get_headers(), ('index0', 'p1', 'p2', 'p3'))
        df = ampl.get_data("1..10", "p1", "p2", "p3")
        self.assertEqual(
            list(df.get_column("p1")),
            [i for i in range(1, 10+1)])
        self.assertEqual(
            list(df.get_column("p2")),
            [2*i for i in range(1, 10+1)])
        self.assertEqual(
            list(df.get_column("p3")),
            [3*i for i in range(1, 10+1)])
        self.assertEqual(df.get_headers(), ('1 .. 10', 'p1', 'p2', 'p3'))
        with self.assertRaises(TypeError):
            ampl.get_data("1..11", "p1", "p2", "p3")

    def test_path(self):
        ampl = self.ampl
        self.assertEqual(
            os.path.abspath(ampl.cd()),
            os.path.abspath(os.curdir)
        )
        ampl.cd('..')
        self.assertEqual(
            os.path.abspath(ampl.cd()),
            os.path.abspath(os.path.join(os.curdir, '..'))
        )

    def test_options(self):
        ampl = self.ampl
        ampl.set_option('a', 's')
        ampl.set_option('b', 123)
        ampl.set_option('c', 1.23)
        ampl.set_option('d', True)
        with self.assertRaises(TypeError):
            ampl.set_option('e', None)
        self.assertEqual(ampl.get_option('a'), 's')
        self.assertEqual(ampl.get_option('b'), 123)
        self.assertEqual(ampl.get_option('c'), 1.23)
        self.assertEqual(ampl.get_option('d'), True)

    def test_handlers(self):
        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def __init__(self):
                self.lastmsg = None
                super(MyOutputHandler, self).__init__()

            def output(self, kind, msg):
                if kind == amplpy.Kind.DISPLAY:
                    print('Display: |{}|'.format(msg))
                self.lastmsg = msg

        class MyErrorHandler(amplpy.ErrorHandler):
            def __init__(self):
                self.last_wrror = None
                self.last_warning = None

            def error(self, exception):
                print(type(exception))
                print('Error:', exception.get_message())
                self.last_error = exception

            def warning(self, exception):
                print(type(exception))
                print('Warning:', exception.get_message())
                self.last_warning = exception

            def check(self):
                pass

        output_handler = MyOutputHandler()
        ampl.set_output_handler(output_handler)
        error_handler = MyErrorHandler()
        ampl.set_error_handler(error_handler)
        self.assertEqual(ampl.get_output_handler(), output_handler)
        self.assertEqual(ampl.get_error_handler(), error_handler)
        ampl.display('1', '2', '3')
        self.assertTrue('1 = 1' in output_handler.lastmsg)
        self.assertTrue('2 = 2' in output_handler.lastmsg)
        self.assertTrue('3 = 3' in output_handler.lastmsg)
        ampl.eval('display X;')
        self.assertEqual(
            str(error_handler.last_warning.get_message()),
            'X is not defined'
        )
        ampl.eval('diy X;')
        self.assertEqual(
            str(error_handler.last_error.get_message()),
            'syntax error'
        )
        self.assertTrue(
            isinstance(error_handler.last_warning.get_source_name(), basestring)
        )
        self.assertTrue(
            isinstance(error_handler.last_warning.get_line_number(), int)
        )
        self.assertTrue(
            isinstance(error_handler.last_warning.get_offset(), int)
        )
        self.assertTrue(
            isinstance(error_handler.last_warning.get_message(), basestring)
        )

    def test_empty_handlers(self):
        ampl = self.ampl
        callback = amplpy.Runnable()
        output_handler = amplpy.OutputHandler()
        error_handler = amplpy.ErrorHandler()
        ampl.set_output_handler(output_handler)
        ampl.set_error_handler(error_handler)
        ampl.eval_async('display 1;', callback)
        ampl.wait()

    def test_broken_handlers(self):
        ampl = self.ampl

        class OutputHandlerRaise(amplpy.OutputHandler):
            def output(self, kind, msg):
                assert False

        class ErrorHandlerIgnore(amplpy.ErrorHandler):
            def error(self, exception):
                assert True

            def warning(self, exception):
                assert True

        class ErrorHandlerRaise(amplpy.ErrorHandler):
            def error(self, exception):
                raise RuntimeError('failed')

            def warning(self, exception):
                raise RuntimeError('failed')

        with self.assertRaises(amplpy.AMPLException):
            ampl.eval('X X;')

        error_handler_ignore = ErrorHandlerIgnore()
        ampl.set_error_handler(error_handler_ignore)
        ampl.eval('X X;')

        error_handler_raise = ErrorHandlerRaise()
        ampl.set_error_handler(error_handler_raise)
        with self.assertRaises(RuntimeError):
            ampl.eval('X X;')

        ampl.set_output_handler(OutputHandlerRaise())
        with self.assertRaises(RuntimeError):
            ampl.eval('display 1;')

    def test_async(self):
        from threading import Lock
        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def output(self, kind, msg):
                pass

        class MyErrorHandler(amplpy.ErrorHandler):
            def __init__(self, mutex1, mutex2):
                self.mutex1 = mutex1
                self.mutex2 = mutex2

            def error(self, exception):
                try:
                    self.mutex1.release()
                except Exception:
                    pass
                try:
                    self.mutex2.release()
                except Exception:
                    pass
                raise exception

            def warning(self, exception):
                print('Warning:', exception.get_message())

        class Callback(amplpy.Runnable):
            def __init__(self, mutex1, mutex2):
                self.ready = False
                self.mutex1 = mutex1
                self.mutex2 = mutex2
                super(Callback, self).__init__()

            def run(self):
                self.mutex2.acquire()
                self.ready = True
                self.mutex1.release()
                self.mutex2.release()

        model = self.str2file('model.mod', '''
            set X;
            set A := 1..10000000;
            param p{i in A} := i;
        ''')
        data = self.str2file('data.dat', '''
            set X := 1, 2, 3;
        ''')

        mutex1 = Lock()
        mutex2 = Lock()
        try:
            callback = Callback(mutex1, mutex2)
            output_handler = MyOutputHandler()
            ampl.set_output_handler(output_handler)
            error_handler = MyErrorHandler(mutex1, mutex2)
            ampl.set_error_handler(error_handler)

            mutex1.acquire()
            mutex2.acquire()
            callback.ready = False
            ampl.read_async(model, callback)
            self.assertFalse(callback.ready)
            mutex2.release()
            mutex1.acquire()
            self.assertTrue(callback.ready)

            mutex2.acquire()
            callback.ready = False
            ampl.read_data_async(data, callback)
            self.assertFalse(callback.ready)
            mutex2.release()
            mutex1.acquire()
            self.assertTrue(callback.ready)

            mutex2.acquire()
            callback.ready = False
            ampl.eval_async('display {i in A: i not in A};', callback)
            ampl.interrupt()
            self.assertFalse(callback.ready)
            mutex2.release()
            mutex1.acquire()
            self.assertTrue(callback.ready)
            self.assertFalse(ampl.is_busy())
            self.assertTrue(ampl.is_running())

        except Exception:
            mutex1.acquire(False)
            mutex1.release()
            mutex2.acquire(False)
            mutex2.release()
            raise

    def test_get_output(self):
        ampl = self.ampl
        self.assertEqual(ampl.get_output('display 5;'), '5 = 5\n\n')
        with self.assertRaises(ValueError):
            ampl.get_output("display 3")
        with self.assertRaises(ValueError):
            ampl.get_output("for {i in 1..10} {")
        self.assertEqual(ampl.get_output(
            'display 5; display 1;'), '5 = 5\n\n1 = 1\n\n')

    def test_export(self):
        ampl = self.ampl
        model = self.str2file('model.mod', '''
            set A;
            set FLOOR;
            set family {FLOOR};
        ''')
        data = self.str2file('data.dat', '''
            set A := 1, 2, 3, 4;
            set FLOOR := 'first' 'second';
            set family['first'] := 'Gutierrez';
            set family['second'] := 'Montoro';
        ''')
        ampl.read(model)
        ampl.read_data(data)
        model2 = self.tmpfile('model2.mod')
        data2 = self.tmpfile('data2.dat')
        ampl.export_model(model2)
        ampl.export_data(data2)
        ampl.reset()
        ampl.read(model2)
        ampl.read_data(data2)
        self.assertEqual(ampl.set['family']['first'].get_values().to_list(),
                         ['Gutierrez'])
        self.assertEqual(ampl.set['family']['second'].get_values().to_list(),
                         ['Montoro'])
        self.assertEqual(ampl.set['A'].get_values().to_list(), [1, 2, 3, 4])

    def test_pathlib(self):
        ampl = self.ampl
        try:
            from pathlib import Path
        except ImportError:
            return
        model = self.str2file('model.mod', 'set A := 1..10;')
        p = Path(model)
        ampl.read(p)
        ampl.display('A')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
import sys
from threading import Thread, Lock

# from builtins import map, range, object, zip, sorted
from builtins import map, object
from past.builtins import basestring

from .errorhandler import ErrorHandler
from .outputhandler import OutputHandler
from .objective import Objective
from .variable import Variable
from .constraint import Constraint
from .set import Set
from .parameter import Parameter
from .dataframe import DataFrame
from .iterators import EntityMap
from .exceptions import AMPLException
from .entity import Entity
from . import amplpython
try:
    import pandas as pd
except ImportError:
    pd = None
inf = float('inf')


class AMPL(object):
    """An AMPL translator.

    An object of this class can be used to do the following tasks:

    - Run AMPL code. See :func:`~amplpy.AMPL.eval` and
      :func:`~amplpy.AMPL.eval_async` / :func:`~amplpy.AMPL.evalAsync`.
    - Read models and data from files. See :func:`~amplpy.AMPL.read`,
      :func:`~amplpy.AMPL.read_data` / :func:`~amplpy.AMPL.readData`,
      :func:`~amplpy.AMPL.read_async` / :func:`~amplpy.AMPL.readAsync`, and
      :func:`~amplpy.AMPL.read_data_async` / :func:`~amplpy.AMPL.readDataAsync`.
    - Solve optimization problems constructed from model and data (see
      :func:`~amplpy.AMPL.solve` and :func:`~amplpy.AMPL.solve_async` / :func:`~amplpy.AMPL.solveAsync`).
    - Access single Elements of an optimization problem. See
      :func:`~amplpy.AMPL.get_variable` / :func:`~amplpy.AMPL.getVariable`,
      :func:`~amplpy.AMPL.get_constraint` / :func:`~amplpy.AMPL.getConstraint`,
      :func:`~amplpy.AMPL.get_objective` / :func:`~amplpy.AMPL.getObjective`,
      :func:`~amplpy.AMPL.get_set` / :func:`~amplpy.AMPL.getSet`,
      and :func:`~amplpy.AMPL.get_parameter` / :func:`~amplpy.AMPL.getParameter`.
    - Access lists of available entities of an optimization problem. See
      :func:`~amplpy.AMPL.get_variables` / :func:`~amplpy.AMPL.getVariables`,
      :func:`~amplpy.AMPL.get_constraints` / :func:`~amplpy.AMPL.getConstraints`,
      :func:`~amplpy.AMPL.get_objectives` / :func:`~amplpy.AMPL.getObjectives`,
      :func:`~amplpy.AMPL.get_sets` / :func:`~amplpy.AMPL.getSets`,
      and :func:`~amplpy.AMPL.get_parameters` / :func:`~amplpy.AMPL.getParameters`.

    Error handling is two-faced:

    - Errors coming from the underlying AMPL translator (e.g. syntax errors and
      warnings obtained calling the eval method) are handled by
      the :class:`~amplpy.ErrorHandler` which can be set and get via
      :func:`~amplpy.AMPL.get_error_handler` / :func:`~amplpy.AMPL.getErrorHandler` and
      :func:`~amplpy.AMPL.set_error_handler` / :func:`~amplpy.AMPL.setErrorHandler`.
    - Generic errors coming from misusing the API, which are detected in
      Python, are thrown as exceptions.

    The default implementation of the error handler throws exceptions on errors
    and prints to console on warnings.

    The output of every user interaction with the underlying translator is
    handled implementing the abstract class :class:`~amplpy.OutputHandler`.
    The (only) method is called at each block of output from the translator.
    The current output handler can be accessed and set via
    :func:`~amplpy.AMPL.get_output_handler` / :func:`~amplpy.AMPL.getOutputHandler` and
    :func:`~amplpy.AMPL.set_output_handler` / :func:`~amplpy.AMPL.setOutputHandler`.
    """

    def __init__(self, environment=None, langext=None):
        """
        Constructor:
        creates a new AMPL instance with the specified environment if provided.

        Args:
            environment (:class:`~amplpy.Environment`): This allows the user to
            specify the location of the AMPL binaries to be used and to modify
            the environment variables in which the AMPL interpreter will run.

        Raises:
            RuntimeError: If no valid AMPL license has been found or if the
            translator cannot be started for any other reason.
        """
        if environment is None:
            try:
                self._impl = amplpython.AMPL()
            except RuntimeError as exp:
                if str(exp).startswith('AMPL could not be started'):
                    message = (
                        '''Please make sure that the AMPL folder is in '''
                        '''the system search path, or\n'''
                        '''specify the path via:\n'''
                        '''    AMPL(Environment('full path to the AMPL '''
                        '''installation directory'))'''
                    )
                    print('*' * 79, file=sys.stderr)
                    for line in message.split('\n'):
                        print('* {:75} *'.format(line), file=sys.stderr)
                    print('*' * 79, file=sys.stderr)
                raise
        else:
            self._impl = amplpython.AMPL(environment._impl)
        self._error_handler = None
        self._output_handler = None
        self._lock = Lock()
        self._langext = langext
        self.set_output_handler(OutputHandler())
        self.set_error_handler(ErrorHandler())

    def __del__(self):
        """
        Default destructor:
        releases all the resources related to the AMPL instance (most notably
        kills the underlying  interpreter).
        """
        self.close()

    def get_data(self, *statements):
        """
        Get the data corresponding to the display statements. The statements
        can be AMPL expressions, or entities. It captures the equivalent of the
        command:

        .. code-block:: ampl

            display ds1, ..., dsn;

        where ds1, ..., dsn are the ``displayStatements`` with which the
        function is called.

        As only one DataFrame is returned, the operation will fail if the
        results of the display statements cannot be indexed over the same set.
        As a result, any attempt to get data from more than one set, or to get
        data for multiple parameters with a different number of indexing sets
        will fail.

        Args:
            statements: The display statements to be fetched.

        Raises:
            TypeError: if the AMPL visualization command does not succeed
            for one of the reasons listed above.

        Returns:
            DataFrame capturing the output of the display
            command in tabular form.
        """
        return DataFrame._from_data_frame_ref(
            self._impl.getData(list(statements), len(statements))
        )

    def get_entity(self, name):
        """
        Get entity corresponding to the specified name (looks for it in all
        types of entities).

        Args:
            name: Name of the entity.

        Raises:
            TypeError: if the specified entity does not exist.

        Returns:
            The AMPL entity with the specified name.
        """
        return Entity(self._impl.getEntity(name))

    def get_variable(self, name):
        """
        Get the variable with the corresponding name.

        Args:
            name: Name of the variable to be found.

        Raises:
            TypeError: if the specified variable does not exist.
        """
        return Variable(self._impl.getVariable(name))

    def get_constraint(self, name):
        """
        Get the constraint with the corresponding name.

        Args:
            name: Name of the constraint to be found.

        Raises:
            TypeError: if the specified constraint does not exist.
        """
        return Constraint(self._impl.getConstraint(name))

    def get_objective(self, name):
        """
         Get the objective with the corresponding name.

         Args:
            name: Name of the objective to be found.

        Raises:
            TypeError: if the specified objective does not exist.
        """
        return Objective(self._impl.getObjective(name))

    def get_set(self, name):
        """
        Get the set with the corresponding name.

        Args:
            name: Name of the set to be found.

        Raises:
            TypeError: if the specified set does not exist.
        """
        return Set(self._impl.getSet(name))

    def get_parameter(self, name):
        """
        Get the parameter with the corresponding name.

        Args:
            name: Name of the parameter to be found.

        Raises:
            TypeError: if the specified parameter does not exist.
        """
        return Parameter(self._impl.getParameter(name))

    def eval(self, statements, **kwargs):
        """
        Parses AMPL code and evaluates it as a possibly empty sequence of AMPL
        declarations and statements.

        As a side effect, it invalidates all entities (as the passed statements
        can contain any arbitrary command); the lists of entities will be
        re-populated lazily (at first access)

        The output of interpreting the statements is passed to the current
        OutputHandler (see getOutputHandler and
        setOutputHandler).

        By default, errors and warnings are printed on stdout.
        This behavior can be changed reassigning an ErrorHandler
        using setErrorHandler.

        Args:
          statements: A collection of AMPL statements and declarations to
          be passed to the interpreter.

        Raises:
          RuntimeError: if the input is not a complete AMPL statement (e.g.
          if it does not end with semicolon) or if the underlying
          interpreter is not running.
        """
        if self._langext is not None:
            statements = self._langext.translate(statements, **kwargs)
        self._impl.eval(statements)
        self._error_handler_wrapper.check()

    def get_output(self, statements):
        """
        Equivalent to :func:`~amplpy.AMPL.eval` but returns the output as a
        string.

        Args:
          statements: A collection of AMPL statements and declarations to
          be passed to the interpreter.

        Returns:
          A string with the output.
        """
        return self._impl.getOutput(statements)

    def reset(self):
        """
        Clears all entities in the underlying AMPL interpreter, clears all maps
        and invalidates all entities.
        """
        self._impl.reset()

    def close(self):
        """
        Stops the underlying engine, and release all any further attempt to
        execute optimization commands without restarting it will throw an
        exception.
        """
        try:
            self._impl.close()
        except AttributeError:
            pass

    def is_running(self):
        """
        Returns true if the underlying engine is running.
        """
        return self._impl.isRunning()

    def is_busy(self):
        """
        Returns true if the underlying engine is doing an async operation.
        """
        # return self._impl.isBusy()
        if self._lock.acquire(False):
            self._lock.release()
            return False
        return True

    def solve(self):
        """
        Solve the current model.

        Raises:
            RuntimeError: if the underlying interpreter is not running.
        """
        self._impl.solve()

    def read_async(self, filename, callback=None, **kwargs):
        """
        Interprets the specified file asynchronously, interpreting it as a
        model or a script file. As a side effect, it invalidates all entities
        (as the passed file can contain any arbitrary command); the lists of
        entities will be re-populated lazily (at first access).

        Args:
            filename: Path to the file (Relative to the current working
            directory or absolute).

            callback: Callback to be executed when the file has been
            interpreted.
        """
        filename = str(filename)
        if self._langext is not None:
            with open(filename, 'r') as fin:
                newmodel = self._langext.translate(fin.read(), **kwargs)
                with open(filename+'.translated', 'w') as fout:
                    fout.write(newmodel)
                    filename += '.translated'

        def async_call():
            self._lock.acquire()
            try:
                self._impl.read(filename)
                self._error_handler_wrapper.check()
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                if callback is not None:
                    callback.run()
        Thread(target=async_call).start()

    def read_data_async(self, filename, callback=None):
        """
        Interprets the specified data file asynchronously. When interpreting is
        over, the specified callback is called. The file is interpreted as
        data. As a side effect, it invalidates all entities (as the passed file
        can contain any arbitrary command); the lists of entities will be
        re-populated lazily (at first access)

        Args:
            filename: Full path to the file.

            callback: Callback to be executed when the file has been
            interpreted.
        """
        filename = str(filename)

        def async_call():
            self._lock.acquire()
            try:
                self._impl.readData(filename)
                self._error_handler_wrapper.check()
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                if callback is not None:
                    callback.run()
        Thread(target=async_call).start()

    def eval_async(self, statements, callback=None, **kwargs):
        """
        Interpret the given AMPL statement asynchronously.

        Args:
          statements: A collection of AMPL statements and declarations to
          be passed to the interpreter.

          callback: Callback to be executed when the statement has been
          interpreted.

        Raises:
          RuntimeError: if the input is not a complete AMPL statement (e.g.
          if it does not end with semicolon) or if the underlying
          interpreter is not running.
        """
        if self._langext is not None:
            statements = self._langext.translate(
                statements, **kwargs)

        def async_call():
            self._lock.acquire()
            try:
                self._impl.eval(statements)
                self._error_handler_wrapper.check()
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                if callback is not None:
                    callback.run()
        Thread(target=async_call).start()

    def solve_async(self, callback=None):
        """
        Solve the current model asynchronously.

        Args:
          callback: Callback to be executed when the solver is done.
        """
        def async_call():
            self._lock.acquire()
            try:
                self._impl.solve()
            except Exception:
                self._lock.release()
                raise
            else:
                self._lock.release()
                if callback is not None:
                    callback.run()
        Thread(target=async_call).start()

    def wait(self):
        """
        Wait for the current async operation to finish.
        """
        self._lock.acquire()
        self._lock.release()

    def interrupt(self):
        """
        Interrupt an underlying asynchronous operation (execution of AMPL code
        by the AMPL interpreter). An asynchronous operation can be started via
        evalAsync(), solveAsync(), readAsync() and readDataAsync().
        Does nothing if the engine and the solver are idle.
        """
        self._impl.interrupt()

    def cd(self, path=None):
        """
        Get or set the current working directory from the underlying
        interpreter (see https://en.wikipedia.org/wiki/Working_directory).

        Args:
            path: New working directory or None (to display the working
            directory).

        Returns:
            Current working directory.
        """
        if path is None:
            return self._impl.cd()
        return self._impl.cd(path)

    def set_option(self, name, value):
        """
        Set an AMPL option to a specified value.

        Args:
            name: Name of the option to be set (alphanumeric without spaces).

            value: The value the option must be set to.

        Raises:
            InvalidArgumet: if the option name is not valid.

            TypeError: if the value has an invalid type.
        """
        if isinstance(value, bool):
            self._impl.setBoolOption(name, value)
        elif isinstance(value, int):
            self._impl.setIntOption(name, value)
        elif isinstance(value, float):
            self._impl.setDblOption(name, value)
        elif isinstance(value, basestring):
            self._impl.setOption(name, value)
        else:
            raise TypeError

    def get_option(self, name):
        """
         Get the current value of the specified option. If the option does not
         exist, returns None.

         Args:
            name: Option name.

        Returns:
            Value of the option.

        Raises:
            InvalidArgumet: if the option name is not valid.
        """
        try:
            value = self._impl.getOption(name).value()
        except RuntimeError:
            return None
        else:
            try:
                return int(value)
            except ValueError:
                try:
                    return float(value)
                except ValueError:
                    return value

    def read(self, filename, **kwargs):
        """
        Interprets the specified file (script or model or mixed) As a side
        effect, it invalidates all entities (as the passed file can contain any
        arbitrary command); the lists of entities will be re-populated lazily
        (at first access).

        Args:
            filename: Full path to the file.

        Raises:
            RuntimeError: in case the file does not exist.
        """
        filename = str(filename)
        if self._langext is not None:
            with open(filename, 'r') as fin:
                newmodel = self._langext.translate(fin.read(), **kwargs)
                with open(filename+'.translated', 'w') as fout:
                    fout.write(newmodel)
                    filename += '.translated'
        self._impl.read(filename)
        self._error_handler_wrapper.check()

    def read_data(self, filename):
        """
        Interprets the specified file as an AMPL data file. As a side effect,
        it invalidates all entities (as the passed file can contain any
        arbitrary command); the lists of entities will be re-populated lazily
        (at first access). After reading the file, the interpreter is put back
        to "model" mode.

        Args:
            filename: Full path to the file.

        Raises:
            RuntimeError: in case the file does not exist.
        """
        filename = str(filename)
        self._impl.readData(filename)
        self._error_handler_wrapper.check()

    def get_value(self, scalar_expression):
        """
        Get a scalar value from the underlying AMPL interpreter, as a double or
        a string.

        Args:
            scalar_expression: An AMPL expression which evaluates to a scalar
            value.

        Returns:
            The value of the expression.
        """
        return self._impl.getValue(scalar_expression)

    def set_data(self, data, set_name=None):
        """
        Assign the data in the dataframe to the AMPL entities with the names
        corresponding to the column names.

        Args:
            data: The dataframe containing the data to be assigned.

            set_name: The name of the set to which the indices values of the
            DataFrame are to be assigned.

        Raises:
            AMPLException: if the data assignment procedure was not successful.
        """
        if not isinstance(data, DataFrame):
            if pd is not None and isinstance(data, pd.DataFrame):
                data = DataFrame.from_pandas(data)
        if set_name is None:
            self._impl.setData(data._impl)
        else:
            self._impl.setData(data._impl, set_name)

    def read_table(self, table_name):
        """
        Read the table corresponding to the specified name, equivalent to the
        AMPL statement:

        .. code-block:: ampl

            read table table_name;

        Args:
            table_name: Name of the table to be read.
        """
        self._impl.readTable(table_name)

    def write_table(self, table_name):
        """
        Write the table corresponding to the specified name, equivalent to the
        AMPL statement

        .. code-block:: ampl

            write table table_name;

        Args:
            table_name: Name of the table to be written.
        """
        self._impl.writeTable(table_name)

    def display(self, *ampl_expressions):
        """
        Writes on the current OutputHandler the outcome of the AMPL statement.

        .. code-block:: ampl

            display e1, e2, .., en;

        where e1, ..., en are the strings passed to the procedure.

        Args:
            ampl_expressions: Expressions to be evaluated.
        """
        exprs = list(map(str, ampl_expressions))
        self._impl.displayLst(exprs, len(exprs))

    def set_output_handler(self, output_handler):
        """
        Sets a new output handler.

        Args:
            output_handler: The function handling the AMPL output derived from
            interpreting user commands.
        """
        class OutputHandlerInternal(amplpython.OutputHandler):
            def output(self, kind, msg):
                output_handler.output(kind, msg)

        self._output_handler = output_handler
        self._output_handler_internal = OutputHandlerInternal()
        self._impl.setOutputHandler(
            self._output_handler_internal
        )

    def set_error_handler(self, error_handler):
        """
        Sets a new error handler.

        Args:
            error_handler: The object handling AMPL errors and warnings.
        """
        class ErrorHandlerWrapper(ErrorHandler):
            def __init__(self, error_handler):
                self.error_handler = error_handler
                self.last_exception = None

            def error(self, exception):
                if isinstance(exception, amplpython.AMPLException):
                    exception = AMPLException(exception)
                try:
                    self.error_handler.error(exception)
                except Exception as exp:
                    self.last_exception = exp

            def warning(self, exception):
                if isinstance(exception, amplpython.AMPLException):
                    exception = AMPLException(exception)
                try:
                    self.error_handler.warning(exception)
                except Exception as exp:
                    self.last_exception = exp

            def check(self):
                if isinstance(self.last_exception, Exception):
                    exp = self.last_exception
                    self.last_exception = None
                    raise exp

        error_handler_wrapper = ErrorHandlerWrapper(error_handler)

        class InnerErrorHandler(amplpython.ErrorHandler):
            def error(self, exception):
                error_handler_wrapper.error(exception)

            def warning(self, exception):
                error_handler_wrapper.warning(exception)

        self._error_handler = error_handler
        self._error_handler_inner = InnerErrorHandler()
        self._error_handler_wrapper = error_handler_wrapper
        self._impl.setErrorHandler(self._error_handler_inner)

    def get_output_handler(self):
        """
        Get the current output handler.

        Returns:
            The current output handler.
        """
        return self._output_handler

    def get_error_handler(self):
        """
        Get the current error handler.

        Returns:
            The current error handler.
        """
        return self._error_handler

    def get_variables(self):
        """
        Get all the variables declared.
        """
        variables = self._impl.getVariables()
        return EntityMap(variables, Variable)

    def get_constraints(self):
        """
        Get all the constraints declared.
        """
        constraints = self._impl.getConstraints()
        return EntityMap(constraints, Constraint)

    def get_objectives(self):
        """
        Get all the objectives declared.
        """
        objectives = self._impl.getObjectives()
        return EntityMap(objectives, Objective)

    def get_sets(self):
        """
        Get all the sets declared.
        """
        sets = self._impl.getSets()
        return EntityMap(sets, Set)

    def get_parameters(self):
        """
        Get all the parameters declared.
        """
        parameters = self._impl.getParameters()
        return EntityMap(parameters, Parameter)

    def get_current_objective(self):
        """
        Get the the current objective. Returns `None` if no objective is set.
        """
        name = self._impl.getCurrentObjectiveName()
        if name == '':
            return None
        else:
            return self.get_objective(name)

    def _var(self):
        """
        Get/Set a variable.
        """
        class Variables(object):
            def __init__(self, ampl):
                self.ampl = ampl

            def __getitem__(self, name):
                return self.ampl.get_variable(name)

            def __setitem__(self, name, value):
                if isinstance(value, (float, int, basestring)):
                    self.ampl.get_variable(name).set_value(value)
                else:
                    self.ampl.get_variable(name).set_values(value)

            def __iter__(self):
                return self.ampl.get_variables()

        return Variables(self)

    def _con(self):
        """
        Get/Set a constraint.
        """
        class Constraints(object):
            def __init__(self, ampl):
                self.ampl = ampl

            def __getitem__(self, name):
                return self.ampl.get_constraint(name)

            def __setitem__(self, name, value):
                self.ampl.get_constraint(name).set_dual(value)

            def __iter__(self):
                return self.ampl.get_constraints()

        return Constraints(self)

    def _obj(self):
        """
        Get an objective.
        """
        class Objectives(object):
            def __init__(self, ampl):
                self.ampl = ampl

            def __getitem__(self, name):
                return self.ampl.get_objective(name)

            def __iter__(self):
                return self.ampl.get_objectives()

        return Objectives(self)

    def _set(self):
        """
        Get/Set a set.
        """
        class Sets(object):
            def __init__(self, ampl):
                self.ampl = ampl

            def __getitem__(self, name):
                return self.ampl.get_set(name)

            def __setitem__(self, name, values):
                self.ampl.get_set(name).set_values(values)

            def __iter__(self):
                return self.ampl.get_sets()

        return Sets(self)

    def _param(self):
        """
        Get/Set a parameter.
        """
        class Parameters(object):
            def __init__(self, ampl):
                self.ampl = ampl

            def __getitem__(self, name):
                return self.ampl.get_parameter(name)

            def __setitem__(self, name, value):
                if isinstance(value, (float, int, basestring)):
                    self.ampl.get_parameter(name).set(value)
                else:
                    self.ampl.get_parameter(name).set_values(value)

            def __iter__(self):
                return self.ampl.get_parameters()

        return Parameters(self)

    def _option(self):
        """
        Get/Set an option.
        """
        class Options(object):
            def __init__(self, ampl):
                self.ampl = ampl

            def __getitem__(self, name):
                return self.ampl.get_option(name)

            def __setitem__(self, name, value):
                self.ampl.set_option(name, value)

        return Options(self)

    var = property(_var)
    con = property(_con)
    obj = property(_obj)
    set = property(_set)
    param = property(_param)
    option = property(_option)

    def export_model(self, modfile):
        """
        Create a .mod file with the model that has been loaded.

        Args:
            modfile: Path to the file (Relative to the current working
            directory or absolute).
        """
        self._impl.exportModel(modfile)

    def export_data(self, datfile):
        """
        Create a .dat file with the data that has been loaded.

        Args:
            datfile: Path to the file (Relative to the current working
            directory or absolute).
        """
        self._impl.exportData(datfile)

    def _start_recording(self, filename):
        """
        Start recording the session to a file for debug purposes.
        """
        filename = str(filename)
        self.set_option('_log_file_name', filename)
        self.set_option('_log_input_only', True)
        self.set_option('_log', True)

    def _stop_recording(self):
        """
        Stop recording the session.
        """
        self.set_option('_log', False)

    def _load_session(self, filename):
        """
        Load a recorded session.
        """
        filename = str(filename)
        try:
            self.eval(open(filename).read())
        except RuntimeError as exp:
            print(exp)

    # Aliases
    _loadSession = _load_session
    _startRecording = _start_recording
    _stopRecording = _stop_recording
    evalAsync = eval_async
    exportData = export_data
    exportModel = export_model
    getConstraint = get_constraint
    getConstraints = get_constraints
    getCurrentObjective = get_current_objective
    getData = get_data
    getEntity = get_entity
    getErrorHandler = get_error_handler
    getObjective = get_objective
    getObjectives = get_objectives
    getOption = get_option
    getOutput = get_output
    getOutputHandler = get_output_handler
    getParameter = get_parameter
    getParameters = get_parameters
    getSet = get_set
    getSets = get_sets
    getValue = get_value
    getVariable = get_variable
    getVariables = get_variables
    isBusy = is_busy
    isRunning = is_running
    readAsync = read_async
    readData = read_data
    readDataAsync = read_data_async
    readTable = read_table
    setData = set_data
    setErrorHandler = set_error_handler
    setOption = set_option
    setOutputHandler = set_output_handler
    solveAsync = solve_async
    writeTable = write_table



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import object
# from past.builtins import basestring


class BaseClass(object):
    def __init__(self, _impl):
        self._impl = _impl

    def to_string(self):
        return self._impl.toString()

    def __str__(self):
        return self.to_string()

    # Aliases
    toString = to_string



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
# from past.builtins import basestring


class AMPLException(Exception):
    """
    Represent an exception generated by AMPL.
    """

    def __init__(self, _impl):
        super(AMPLException, self).__init__()
        self._impl = _impl
        self.what = self._impl.what()
        self.source_name = _impl.getSourceName()
        self.line_number = _impl.getLineNumber()
        self.offset = _impl.getOffset()
        self.message = _impl.getMessage()

    def get_source_name(self):
        """
        Get the name of the file where the error was detected.
        """
        return self.source_name

    def get_line_number(self):
        """
        Get the row where the error is located.
        """
        return self.line_number

    def get_offset(self):
        """
        Get the offset where the error is located.
        """
        return self.offset

    def get_message(self):
        """
        Get the error message.
        """
        return self.message

    def __str__(self):
        return self.what.lstrip('file: -').strip('\n')

    # Aliases
    getLineNumber = get_line_number
    getMessage = get_message
    getOffset = get_offset
    getSourceName = get_source_name



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import map
from numbers import Real
from past.builtins import basestring

from .entity import Entity
from .dataframe import DataFrame
from .iterators import MemberRangeIterator
try:
    import numpy as np
except ImportError:
    np = None


class Set(Entity):
    """
    Represents an AMPL set. In case of not indexed sets, this class exposes
    iterators for accessing its elements. The members of the set are tuples.
    All these methods throw a TypeError if called for an indexed set.

    In case of indexed sets, you can gain access to the instances using the
    operator [].

    All the accessors in this class throw an RuntimeError if the instance has
    been deleted in the underlying AMPL interpreter.

    Data can be assigned to the set using the methods
    :func:`~amplpy.Set.set_values` / :func:`~amplpy.Set.setValues`
    (for non-indexed sets only) or using
    :func:`~amplpy.AMPL.set_data` / :func:`~amplpy.AMPL.setData`
    and an object of class
    :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        Entity.__init__(self, _impl, Set)

    def instances(self):
        """
        Get an iterator to iterate over all the instances in a Set.
        """
        return Entity.instances(self)

    def arity(self):
        """
        The arity of the set, or number of components in each member of this
        set.
        """
        return int(self._impl.arity())

    def get_values(self):
        """
        Get values of this set in a DataFrame. Valid only for non-indexed sets.
        """
        return Entity.get_values(self)

    def members(self):
        """
        Get members (tuples) of this Set. Valid only for non-indexed sets.
        """
        return MemberRangeIterator(self._impl.members())

    def size(self):
        """
        Get the number of tuples in this set. Valid only for non-indexed sets.
        """
        return int(self._impl.size())

    def contains(self, t):
        """
        Check wether this set instance contains the specified Tuple.
        Valid only for non-indexed sets.

        Args:
            t: Tuple to be found.
        """
        return self._impl.contains(t)

    def set_values(self, values):
        """
        Set the tuples in this set. Valid only for non-indexed sets.

        Args:
            values: A list of tuples or a :class:`~amplpy.DataFrame`.

        In the case of a :class:`~amplpy.DataFrame`, the number of indexing
        columns of the must be equal to the arity of the set. In the case of
        a list of tuples, the arity of each tuple must be equal to the arity
        of the set.

        For example, considering the following AMPL entities and corresponding
        Python objects:

        .. code-block:: ampl

            set A := 1..2;
            param p{i in A} := i+10;
            set AA;

        The following is valid:

        .. code-block:: python

            A, AA = ampl.getSet('A'), ampl.getSet('AA')
            AA.setValues(A.getValues())  # AA has now the members {1, 2}
        """
        if isinstance(values, (list, set)):
            if all(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.setValuesStr(values, len(values))
            elif all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.setValuesDbl(values, len(values))
            else:
                if not isinstance(values, list):
                    values = list(values)
                self._impl.setValuesTuples(values, len(values))
        else:
            if np is not None and isinstance(values, np.ndarray):
                self.set_values(DataFrame.from_numpy(values).to_list())
                return
            Entity.set_values(self, values)

    # Aliases
    getValues = get_values
    setValues = set_values



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import sorted

import unittest
import os


class TestEnvironment(unittest.TestCase):
    """Test Environment."""

    def test_environment_initialization(self):
        from amplpy import Environment
        env = Environment('binary_directory')
        self.assertEqual('binary_directory', env.get_bin_dir())
        env = Environment('binary_directory', 'binary_name')
        self.assertEqual('binary_directory', env.get_bin_dir())
        self.assertEqual('binary_name', env.get_bin_name())
        env.set_bin_dir('binary_directory_2')
        self.assertEqual('binary_directory_2', env.get_bin_dir())
        env.set_bin_name('binary_name_2')
        self.assertEqual('binary_name_2', env.get_bin_name())

    def test_environment(self):
        from amplpy import Environment, AMPL
        env1 = Environment()
        env2 = Environment(os.curdir)
        self.assertEqual(env2.get_bin_dir(), os.curdir)
        env1.set_bin_dir(env2.get_bin_dir())
        self.assertEqual(env1.get_bin_dir(), env1.get_bin_dir())
        self.assertEqual(len(dict(env1)), len(list(env1)))
        self.assertEqual(list(sorted(dict(env1).items())), list(sorted(env1)))
        env1['MyEnvVar'] = 'TEST'
        self.assertEqual(env1['MyEnvVar'], 'TEST')
        self.assertEqual(env2['MyEnvVar'], None)
        d = dict(env1)
        self.assertEqual(d['MyEnvVar'], 'TEST')
        ampl = AMPL(Environment())
        ampl.close()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import object
from past.builtins import basestring


class Iterator(object):
    def __init__(self, obj, function):
        self.obj = obj
        self.iterator = obj.begin()
        self.end = obj.end()
        self.function = function

    def __iter__(self):
        return self

    def __next__(self):
        if self.iterator.equals(self.end):
            raise StopIteration
        to_return = self.function(self.iterator)
        self.iterator.postIncrement()
        return to_return


class EntityMap(Iterator):
    def __init__(self, obj, entity_class):
        self.entity_class = entity_class

        def pair(iterator):
            entity = entity_class(iterator.__ref__())
            return (entity.name(), entity)

        Iterator.__init__(self, obj, pair)

    def __getitem__(self, key):
        assert isinstance(key, basestring)
        return self.entity_class(self.obj.getIndex(key))

    def size(self):
        return int(self.obj.size())

    def __len__(self):
        return self.size()


class InstanceIterator(Iterator):
    def __init__(self, obj, instanceClass):
        Iterator.__init__(
            self, obj,
            lambda it: (
                it.first(),
                instanceClass(it.second())
            ),
        )

    def size(self):
        return int(self.obj.numInstances())

    def __len__(self):
        return self.size()


class MemberRangeIterator(Iterator):
    """Iterator for set members."""

    def __init__(self, obj):
        Iterator.__init__(
            self, obj,
            lambda it: it.__ref__()
        )

    def size(self):
        return int(self.obj.size())

    def __len__(self):
        return self.size()


def EnvIterator(obj):
    """Iterator for environment classes."""
    return Iterator(
        obj,
        lambda it: (it.first(), it.second())
    )


def ColIterator(obj):
    return Iterator(
        obj,
        lambda it: it.__ref__()
    )


def RowIterator(obj):
    return Iterator(
        obj,
        lambda it: it.__ref__()
    )



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import inflection

from .base import BaseClass
from .outputhandler import OutputHandler, Kind
from .errorhandler import ErrorHandler
from .exceptions import AMPLException
from .iterators import EntityMap
from .runnable import Runnable
from .objective import Objective
from .variable import Variable
from .constraint import Constraint
from .set import Set
from .parameter import Parameter
from .entity import Entity
from .dataframe import DataFrame, Row, Column
from .utils import multidict, register_magics
from .environment import Environment
from .ampl import AMPL
__version__ = '0.8.0'


def _list_aliases():
    classes = [
        BaseClass, OutputHandler, ErrorHandler,
        AMPLException, EntityMap, Runnable,
        Entity, Objective, Variable, Constraint, Set, Parameter,
        Row, Column, DataFrame, Environment, AMPL]
    for cls in classes:
        print(cls)
        for method in list(dir(cls)):
            if method.startswith('__'):
                continue
            cammel_method = inflection.camelize(method, False)
            if cammel_method != method:
                print('\t{} = {}'.format(cammel_method, method))
                # setattr(cls, cammel_method, getattr(cls, method))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted

import unittest
from . import TestBase


class TestIterators(TestBase.TestBase):
    """Test Iterators."""

    def test_set_iterators(self):
        ampl = self.ampl
        ampl.eval('set x;')
        self.assertEqual(
            len(ampl.get_sets()),
            ampl.get_sets().size()
        )
        self.assertEqual(
            ampl.get_sets().size(),
            len(list(ampl.get_sets()))
        )
        self.assertEqual(ampl.get_sets().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.get_sets().size(), 0)
        ampl.eval('set x;')
        self.assertEqual(ampl.get_sets().size(), 1)
        ampl.eval('set y{1..2};')
        self.assertEqual(ampl.get_sets().size(), 2)
        ampl.eval('set z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.get_sets().size(), 3)
        ampl.eval('set xx{1..2,4..5};')
        self.assertEqual(ampl.get_sets().size(), 4)
        ampl.eval('set yy{1..2,4..5,7..8};')
        self.assertEqual(ampl.get_sets().size(), 5)
        ampl.eval('set zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.get_sets().size(), 6)
        self.assertEqual(
            ampl.get_set('x').num_instances(),
            len(dict(ampl.get_set('x')))
        )
        self.assertEqual(ampl.get_set('x').num_instances(), 1)
        self.assertEqual(ampl.get_set('y').num_instances(), 2)
        self.assertEqual(ampl.get_set('z').num_instances(), 4)
        self.assertEqual(ampl.get_set('xx').num_instances(), 4)
        self.assertEqual(ampl.get_set('yy').num_instances(), 8)
        self.assertEqual(ampl.get_set('zz').num_instances(), 8)
        self.assertEqual(
            max(s.num_instances() for name, s in ampl.get_sets()), 8
        )
        self.assertEqual(
            ampl.get_set('x').name(),
            ampl.get_sets()['x'].name()
        )

    def test_parameter_iterators(self):
        ampl = self.ampl
        ampl.eval('param x;')
        self.assertEqual(
            len(ampl.get_parameters()),
            ampl.get_parameters().size()
        )
        self.assertEqual(
            ampl.get_parameters().size(),
            len(list(ampl.get_parameters()))
        )
        self.assertEqual(ampl.get_parameters().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.get_parameters().size(), 0)
        ampl.eval('param x;')
        self.assertEqual(ampl.get_parameters().size(), 1)
        ampl.eval('param y{1..2};')
        self.assertEqual(ampl.get_parameters().size(), 2)
        ampl.eval('param z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.get_parameters().size(), 3)
        ampl.eval('param xx{1..2,4..5};')
        self.assertEqual(ampl.get_parameters().size(), 4)
        ampl.eval('param yy{1..2,4..5,7..8};')
        self.assertEqual(ampl.get_parameters().size(), 5)
        ampl.eval('param zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.get_parameters().size(), 6)
        self.assertEqual(
            ampl.get_parameter('x').num_instances(),
            len(dict(ampl.get_parameter('x')))
        )
        self.assertEqual(ampl.get_parameter('x').num_instances(), 1)
        self.assertEqual(ampl.get_parameter('y').num_instances(), 2)
        self.assertEqual(ampl.get_parameter('z').num_instances(), 4)
        self.assertEqual(ampl.get_parameter('xx').num_instances(), 4)
        self.assertEqual(ampl.get_parameter('yy').num_instances(), 8)
        self.assertEqual(ampl.get_parameter('zz').num_instances(), 8)
        self.assertEqual(
            max(p.num_instances() for name, p in ampl.get_parameters()), 8
        )
        self.assertEqual(
            ampl.get_parameter('x').name(),
            ampl.get_parameters()['x'].name()
        )

    def test_variable_iterators(self):
        ampl = self.ampl
        ampl.eval('var x;')
        self.assertEqual(
            len(ampl.get_variables()),
            ampl.get_variables().size()
        )
        self.assertEqual(
            ampl.get_variables().size(),
            len(list(ampl.get_variables()))
        )
        self.assertEqual(ampl.get_variables().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.get_variables().size(), 0)
        ampl.eval('var x;')
        self.assertEqual(ampl.get_variables().size(), 1)
        ampl.eval('var y{1..2};')
        self.assertEqual(ampl.get_variables().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.get_variables().size(), 3)
        ampl.eval('var xx{1..2,4..5};')
        self.assertEqual(ampl.get_variables().size(), 4)
        ampl.eval('var yy{1..2,4..5,7..8};')
        self.assertEqual(ampl.get_variables().size(), 5)
        ampl.eval('var zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.get_variables().size(), 6)
        self.assertEqual(
            ampl.get_variable('x').num_instances(),
            len(dict(ampl.get_variable('x')))
        )
        self.assertEqual(ampl.get_variable('x').num_instances(), 1)
        self.assertEqual(ampl.get_variable('y').num_instances(), 2)
        self.assertEqual(ampl.get_variable('z').num_instances(), 4)
        self.assertEqual(ampl.get_variable('xx').num_instances(), 4)
        self.assertEqual(ampl.get_variable('yy').num_instances(), 8)
        self.assertEqual(ampl.get_variable('zz').num_instances(), 8)
        self.assertEqual(
            max(var.num_instances() for name, var in ampl.get_variables()), 8
        )
        self.assertEqual(
            ampl.get_variable('x').name(),
            ampl.get_variables()['x'].name()
        )

    def test_constraint_iterators(self):
        ampl = self.ampl
        ampl.eval('var x;')
        ampl.eval('s.t. c_x: x = 0;')
        self.assertEqual(
            ampl.get_constraints().size(),
            len(ampl.get_constraints())
        )
        self.assertEqual(
            ampl.get_constraints().size(),
            len(list(ampl.get_constraints()))
        )
        self.assertEqual(ampl.get_constraints().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.get_constraints().size(), 0)
        ampl.eval('var x;')
        ampl.eval('s.t. c_x: x = 0;')
        self.assertEqual(ampl.get_constraints().size(), 1)
        ampl.eval('var y{1..2};')
        ampl.eval('s.t. c_y{i in 1..2}: y[i] = i;')
        self.assertEqual(ampl.get_constraints().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('s.t. c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j] = i;')
        self.assertEqual(ampl.get_constraints().size(), 3)
        ampl.eval('var xx{1..2,4..5};')
        ampl.eval('s.t. c_xx{(i, j) in {1..2,4..5}}: xx[i,j] = i+j;')
        self.assertEqual(ampl.get_constraints().size(), 4)
        self.assertEqual(
            ampl.get_constraint('c_x').num_instances(),
            len(dict(ampl.get_constraint('c_x')))
        )
        self.assertEqual(ampl.get_constraint('c_x').num_instances(), 1)
        self.assertEqual(ampl.get_constraint('c_y').num_instances(), 2)
        self.assertEqual(ampl.get_constraint('c_z').num_instances(), 4)
        self.assertEqual(ampl.get_constraint('c_xx').num_instances(), 4)
        self.assertEqual(
            max(con.num_instances() for name, con in ampl.get_constraints()), 4
        )
        self.assertEqual(
            ampl.get_constraint('c_x').name(),
            ampl.get_constraints()['c_x'].name()
        )

    def test_objective_iterators(self):
        ampl = self.ampl
        ampl.eval('var x;')
        ampl.eval('maximize c_x: x;')
        self.assertEqual(
            ampl.get_objectives().size(),
            len(ampl.get_objectives())
        )
        self.assertEqual(
            ampl.get_objectives().size(),
            len(list(ampl.get_objectives()))
        )
        self.assertEqual(ampl.get_objectives().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.get_objectives().size(), 0)
        ampl.eval('var x;')
        ampl.eval('maximize c_x: x;')
        self.assertEqual(ampl.get_objectives().size(), 1)
        ampl.eval('var y{1..2};')
        ampl.eval('maximize c_y{i in 1..2}: y[i];')
        self.assertEqual(ampl.get_objectives().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('maximize c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j];')
        self.assertEqual(ampl.get_objectives().size(), 3)
        ampl.eval('var xx{1..2,4..5};')
        ampl.eval('maximize c_xx{(i, j) in {1..2,4..5}}: xx[i,j];')
        self.assertEqual(ampl.get_objectives().size(), 4)
        self.assertEqual(
            ampl.get_objective('c_x').num_instances(),
            len(dict(ampl.get_objective('c_x')))
        )
        self.assertEqual(ampl.get_objective('c_x').num_instances(), 1)
        self.assertEqual(ampl.get_objective('c_y').num_instances(), 2)
        self.assertEqual(ampl.get_objective('c_z').num_instances(), 4)
        self.assertEqual(ampl.get_objective('c_xx').num_instances(), 4)
        self.assertEqual(
            max(obj.num_instances() for name, obj in ampl.get_objectives()), 4
        )
        self.assertEqual(
            ampl.get_objective('c_x').name(),
            ampl.get_objectives()['c_x'].name()
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import map, range, zip
from numbers import Real
from past.builtins import basestring

from .base import BaseClass
from .iterators import RowIterator, ColIterator
from . import amplpython
try:
    import pandas as pd
except ImportError:
    pd = None
try:
    import numpy as np
except ImportError:
    np = None


class Row(BaseClass):
    """
    Represents a row in a :class:`~amplpy.DataFrame`.
    """

    def __iter__(self):
        return RowIterator(self._impl)

    def __getitem__(self, key):
        return self._impl.getIndex(key)

    def to_string(self):
        return str(list(self))

    # Aliases
    toString = to_string


class Column(BaseClass):
    """
    Represents a column in a :class:`~amplpy.DataFrame`.
    """

    def __iter__(self):
        return ColIterator(self._impl)

    def to_string(self):
        return str(self.to_list())

    def to_list(self):
        return self._impl.toPyList()

    # Aliases
    toString = to_string
    toList = to_list


class DataFrame(BaseClass):
    """
    A DataFrame object, used to communicate data to and from the AMPL entities.

    An object of this class can be used to do the following tasks:
    - Assign values to AMPL entities (once the DataFrame is populated, use
    :func:`~amplpy.AMPL.set_data` / :func:`~amplpy.AMPL.setData` to assign its
    values to the modelling entities in its columns)
    - Get values from AMPL, decoupling the values from the AMPL entities they
    originate via
    :func:`~amplpy.Entity.get_values` / :func:`~amplpy.Entity.getValues`.

    A DataFrame object can be created in various ways.

    - Create a skeleton by specifiying manually the indexing columns and the
      column headers.
    - Get values from AMPL, decoupling the values from the AMPL entities they
      originate from
      (via :func:`~amplpy.Entity.get_values` / :func:`~amplpy.Entity.getValues`).

    Populating a DataFrame object can be done adding row by row to a
    pre-existing skeleton via :func:`~amplpy.DataFrame.addRow`, setting whole
    columns of a pre-existing skeleton via
    :func:`~amplpy.DataFrame.set_column` / :func:`~amplpy.DataFrame.setColumn`
    or adding columns (including indexing columns) via
    :func:`~amplpy.DataFrame.add_column` / :func:`~amplpy.DataFrame.addColumn`.

    Modifying a DataFrame object can be done via
    :func:`~amplpy.DataFrame.set_column` / :func:`~amplpy.DataFrame.setColumn`
    or, item by item, via
    :func:`~amplpy.DataFrame.set_values` / :func:`~amplpy.DataFrame.setValues`.

    Accessing data in a DataFrame can be done row by row using
    :func:`~amplpy.DataFrame.get_row` / :func:`~amplpy.DataFrame.getRow`
    or by column via
    :func:`~amplpy.DataFrame.get_column` / :func:`~amplpy.DataFrame.getColumn`.
    """

    def __init__(self, index, columns=tuple(), **kwargs):
        """
        Create a new DataFrame with specifed index and column headers.

        Args:
            index: Index column;

            columns: Column headers.
        """
        if index is not None:
            if isinstance(index, basestring):
                index = (index,)
            if isinstance(columns, basestring):
                columns = (columns,)
            index_names = [
                col[0] if isinstance(col, tuple) else col
                for col in index
            ]
            column_names = [
                col[0] if isinstance(col, tuple) else col
                for col in columns
            ]
            impl = amplpython.DataFrame.factory(
                len(index_names),
                list(index_names) + list(column_names),
                len(index_names) + len(column_names)
            )
            super(DataFrame, self).__init__(impl)
            for col in index:
                if isinstance(col, tuple):
                    self.set_column(col[0], col[1])
            for col in columns:
                if isinstance(col, tuple):
                    self.set_column(col[0], col[1])
        else:
            impl = kwargs.get('_impl', None)
            super(DataFrame, self).__init__(impl)

    def __iter__(self):
        # FIXME: C++ iterators for dataframes not working with SWIG.
        return (self.get_row_by_index(i) for i in range(self.get_num_rows()))

    def get_num_cols(self):
        """
        Get the total number of columns in this dataframe (indexarity + number
        of values).

        Returns:
            The number of columns.
        """
        return self._impl.getNumCols()

    def get_num_rows(self):
        """
        Get the number of data rows in this dataframe.

        Returns:
            The number of rows.
        """
        return self._impl.getNumRows()

    def get_num_indices(self):
        """
        Get the number of indices (the indexarity) of this dataframe.

        Returns:
            The number of indices needed to access one row of this dataframe.
        """
        return self._impl.getNumIndices()

    def add_row(self, *value):
        """
         Add a row to the DataFrame. The size of the tuple must be equal to the
         total number of columns in the dataframe.

         Args:
            value: A single argument with a tuple containing all the values
            for the row to be added, or multiple arguments with the values for
            each column.
        """
        if len(value) == 1 and isinstance(value[0], (tuple, list)):
            value = value[0]
        assert len(value) == self.get_num_cols()
        self._impl.addRow(tuple(value))

    def add_column(self, header, values=None):
        """
        Add a new column with the corresponding header and values to the
        dataframe.

        Args:
            header: The name of the new column.

            values: A list of size :func:`~amplpy.DataFrame.getNumRows` with
            all the values of the new column.
        """
        if values is None:
            values = []
        if len(values) == 0:
            self._impl.addColumn(header)
        else:
            assert len(values) == self.get_num_rows()
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.addColumnStr(header, values)
            elif all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.addColumnDbl(header, values)
            else:
                raise NotImplementedError

    def get_column(self, header):
        """
        Get the specified column as a view object.

        Args:
            header: The header of the column.
        """
        return Column(self._impl.getColumn(header))

    def set_column(self, header, values):
        """
        Set the values of a column.

        Args:
            header: The header of the column to be set.

            values: The values to set.
        """
        res = self._impl.setColumnPyList(header, list(values))
        if res != 0:
            raise NotImplementedError
        # if any(isinstance(value, basestring) for value in values):
        #     values = list(map(str, values))
        #     self._impl.setColumnStr(header, values, len(values))
        # elif all(isinstance(value, Real) for value in values):
        #     values = list(map(float, values))
        #     self._impl.setColumnDbl(header, values, len(values))
        # else:
        #     raise NotImplementedError

    def get_row(self, key):
        """
        Get a row by value of the indexing columns. If the index is not
        specified, gets the only row of a dataframe with no indexing columns.

        Args:
            key: Tuple representing the index of the desired row.

        Returns:
            The row.
        """
        return Row(self._impl.getRowTpl(key))

    def get_row_by_index(self, index):
        """
        Get row by numeric index.

        Args:
            index: Zero-based index of the row to get.

        Returns:
            The corresponding row.
        """
        return Row(self._impl.getRowByIndex(index))

    def get_headers(self):
        """
         Get the headers of this DataFrame.

         Returns:
            The headers of this DataFrame.
        """
        return tuple(self._impl.getHeaders())

    def set_values(self, values):
        """
        Set the values of a DataFrame from a dictionary.

        Args:
            values: Dictionary with the values to set.
        """
        ncols = self.get_num_cols()
        nindices = self.get_num_indices()

        def conv_to_list(value):
            if isinstance(value, list):
                return value
            elif isinstance(value, tuple):
                return list(value)
            else:
                return [value]
        for key, value in values.items():
            key = conv_to_list(key)
            assert len(key) == nindices
            value = conv_to_list(value)
            assert len(value) == ncols-nindices
            self.add_row(key + value)

    def to_dict(self):
        """
        Return a dictionary with the DataFrame data.
        """
        d = {}
        nindices = self.get_num_indices()
        if nindices == 0:
            raise ValueError('cannot convert to dictionary without an index')
        data = zip(*[
            self.get_column(header).to_list()
            for header in self.get_headers()
        ])
        for row in data:
            if nindices == 1:
                key = row[0]
            else:
                key = tuple(row[:nindices])
            if len(row) - nindices == 0:
                d[key] = None
            elif len(row) - nindices == 1:
                d[key] = row[nindices]
            else:
                d[key] = tuple(row[nindices:])
        return d

    def to_list(self):
        """
        Return a list with the DataFrame data.
        """
        if self.get_num_cols() > 1:
            return [
                tuple(self.get_row_by_index(i))
                for i in range(self.get_num_rows())
            ]
        else:
            return [
                self.get_row_by_index(i)[0]
                for i in range(self.get_num_rows())
            ]

    def to_pandas(self):
        """
        Return a pandas DataFrame with the DataFrame data.
        """
        assert pd is not None
        nindices = self.get_num_indices()
        headers = self.get_headers()
        columns = {
            header: self.get_column(header).to_list()
            for header in headers[nindices:]
        }
        index = zip(*[
            self.get_column(header).to_list()
            for header in headers[:nindices]
        ])
        index = [key if len(key) > 1 else key[0] for key in index]
        if index == []:
            return pd.DataFrame(columns, index=None)
        else:
            return pd.DataFrame(columns, index=index)

    @classmethod
    def from_dict(cls, dic, index_names=None, column_names=None):
        """
        Create a :class:`~amplpy.DataFrame` from a dictionary.

        Args:
            dic: dictionary to load.
            index_names: index names to use.
            column_names: column names to use.
        """
        assert isinstance(dic, dict)
        assert len(dic) != 0

        def to_tuple(e):
            if isinstance(e, (tuple, list)):
                return tuple(e)
            else:
                return (e,)

        lst_index = list(map(to_tuple, dic.keys()))
        lst_columns = list(map(to_tuple, dic.values()))
        nindices, ncolumns = len(lst_index[0]), len(lst_columns[0])
        assert index_names is None or nindices == len(index_names)
        assert column_names is None or ncolumns == len(column_names)
        assert all(len(k) == nindices for k in lst_index)
        assert all(len(v) == ncolumns for v in lst_columns)

        index = zip(*lst_index)
        columns = zip(*lst_columns)

        if index_names is None:
            index_names = ['index{}'.format(i) for i in range(nindices)]

        if column_names is None:
            column_names = ['value{}'.format(i) for i in range(ncolumns)]

        index = [
            (index_names[i], cindex)
            for i, cindex in enumerate(zip(*lst_index))
        ]
        columns = [
            (column_names[i], column)
            for i, column in enumerate(zip(*lst_columns))
        ]
        return cls(index=index, columns=columns)

    @classmethod
    def from_pandas(cls, df, index_names=None):
        """
        Create a :class:`~amplpy.DataFrame` from a pandas DataFrame.

        Args:
            df: Pandas DataFrame to load.
            index_names: index names to use.
        """
        assert pd is not None
        if isinstance(df, pd.Series):
            df = pd.DataFrame(df)
        else:
            assert isinstance(df, pd.DataFrame)
        keys = [
            key if isinstance(key, tuple) else (key,)
            for key in df.index.tolist()
        ]
        index = [
            ('index{}'.format(i), cindex)
            for i, cindex in enumerate(zip(*keys))
        ]
        if index_names is not None:
            assert len(index) == len(index_names)
            for i in range(len(index)):
                index[i] = (index_names[i], index[i][1])
        columns = [
            (str(cname), df[cname].tolist())
            for cname in df.columns.tolist()
        ]
        return cls(index=index, columns=columns)

    @classmethod
    def from_numpy(cls, data):
        """
        Create a :class:`~amplpy.DataFrame` from a numpy array or matrix.
        """
        assert np is not None
        if isinstance(data, np.ndarray):
            index = []
            if len(data.shape) == 1:
                columns = [('value', data.tolist())]
            elif len(data.shape) == 2:
                columns = [
                    ('c{}'.format(i), col)
                    for i, col in enumerate(zip(*data.tolist()))
                ]
            else:
                raise TypeError
        else:
            raise TypeError
        return cls(index=index, columns=columns)

    @classmethod
    def _from_data_frame_ref(cls, df_ref):
        return cls(None, None, _impl=df_ref)

    # Aliases
    addColumn = add_column
    addRow = add_row
    fromDict = from_dict
    fromNumpy = from_numpy
    fromPandas = from_pandas
    getColumn = get_column
    getHeaders = get_headers
    getNumCols = get_num_cols
    getNumIndices = get_num_indices
    getNumRows = get_num_rows
    getRow = get_row
    getRowByIndex = get_row_by_index
    setColumn = set_column
    setValues = set_values
    toDict = to_dict
    toList = to_list
    toPandas = to_pandas



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import range

import unittest
import string
import random
import os
from amplpy import DataFrame
from . import TestBase


class TestDataFrame(TestBase.TestBase):
    """Test DataFrame."""

    def test_dataframe(self):
        # Create first dataframe (for data indexed over NUTR)
        # Add data row by row
        df1 = DataFrame('NUTR', ('n_min', 'n_max'))
        df1.add_row(('A', 700, 20000))
        df1.add_row(('B1', 700, 20000))
        df1.add_row(('B2', 700, 20000))
        df1.add_row(('C', 700, 20000))
        df1.add_row(('CAL', 16000, 24000))
        df1.add_row(('NA', 0.0, 50000))

        # Create second dataframe (for data indexed over FOOD)
        # Add column by column
        df2 = DataFrame('FOOD')
        foods = ['BEEF', 'CHK', 'FISH', 'HAM', 'MCH', 'MTL', 'SPG', 'TUR']
        df2.set_column('FOOD', foods)
        self.assertEqual(list(df2.get_column('FOOD')), foods)
        contents = [2] * 8
        df2.add_column('f_min', contents)
        self.assertEqual(list(df2.get_column('f_min')), contents)
        contents = [10] * 8
        df2.add_column('f_max', contents)
        self.assertEqual(list(df2.get_column('f_max')), contents)
        costs = [3.19, 2.59, 2.29, 2.89, 1.89, 1.99, 1.99, 2.49]
        df2.add_column('cost', costs)
        self.assertEqual(list(df2.get_column('cost')), costs)
        labels = [random.choice(string.ascii_letters)] * 8
        df2.add_column('labels', labels)
        self.assertEqual(list(df2.get_column('labels')), labels)
        df2.add_column('empty', [])
        self.assertEqual(list(df2.get_column('empty')), [None] * 8)

        print(df2.get_column('FOOD'))
        for index in df2.get_column('FOOD'):
            print(df2.get_row(index))

        # Create third dataframe, to assign data to the AMPL entity
        # param amt{NUTR, FOOD};
        df3 = DataFrame(('NUTR', 'FOOD'))
        # Populate the set columns
        nutr_with_multiplicity = ['']*48
        food_with_multiplicity = ['']*48
        i = 0
        for n in range(6):
            for f in range(8):
                print(df1.get_row_by_index(n)[0])
                nutr_with_multiplicity[i] = df1.get_row_by_index(n)[0]
                food_with_multiplicity[i] = foods[f]
                i += 1
        df3.set_column('NUTR', nutr_with_multiplicity)
        df3.set_column('FOOD', food_with_multiplicity)

        # Populate with all these values
        values = [
            60, 8, 8, 40, 15, 70, 25, 60, 10, 20, 15, 35, 15, 15, 25, 15, 15,
            20, 10, 10, 15, 15, 15, 10, 20, 0, 10, 40, 35, 30, 50, 20, 295,
            770, 440, 430, 315, 400, 370, 450, 968, 2180, 945, 278, 1182, 896,
            1329, 1397
        ]
        df3.add_column('amt', values)

    def test_pandas(self):
        ampl = self.ampl
        try:
            import pandas as pd
        except ImportError:
            return
        df = pd.DataFrame({
            'a': [1, 2],
            'b': [3.5, 4]
        },
            index=['x', 'y']
        )
        ampl.eval('''
            set S;
            param a{S};
            param b{S};
        ''')
        ampl.set_data(df, 'S')
        self.assertEqual(list(ampl.set['S'].members()), ['x', 'y'])
        self.assertEqual(ampl.param['a']['x'], 1)
        self.assertEqual(ampl.param['b']['y'], 4)

        df2 = pd.DataFrame({
            'a': [10, 20, 30],
        },
            index=['x', 'y', 'z']
        )
        df3 = pd.DataFrame({}, index=['xx', 'yy'])
        df = DataFrame.from_pandas(df)
        df2 = DataFrame.from_pandas(df2)
        df3 = DataFrame.from_pandas(df3)
        self.assertTrue(isinstance(df.to_dict(), dict))
        self.assertTrue(isinstance(df.to_list(), list))
        self.assertTrue(isinstance(df.to_pandas(), pd.DataFrame))

        self.assertEqual(df.to_list()[0][1:], (1, 3.5))
        self.assertEqual(df2.to_list()[0], ('x', 10))
        self.assertEqual(df3.to_list()[0], 'xx')

        self.assertEqual(set(df.to_dict().keys()), set(['x', 'y']))
        self.assertEqual(set(df2.to_dict().keys()), set(['x', 'y', 'z']))
        self.assertEqual(set(df3.to_dict().keys()), set(['xx', 'yy']))

        self.assertEqual(df.to_dict()['x'], (1, 3.5))
        self.assertEqual(df2.to_dict()['x'], 10)
        self.assertEqual(df3.to_dict()['xx'], None)

        csv_file = os.path.join(os.path.dirname(__file__), 'data.csv')
        p_df = pd.read_csv(csv_file, sep=';', index_col=0)
        df = DataFrame.from_pandas(p_df)
        self.assertTrue(isinstance(df.to_dict(), dict))
        self.assertEqual(set(df.to_dict().keys()), set([1.0, 2.0, 3.0]))
        self.assertEqual(set(df.to_list()[0]), set([1.0, 0.01]))
        self.assertEqual(set(df.to_list()[1]), set([2.0, 0.02]))
        self.assertEqual(set(df.to_list()[2]), set([3.0, 0.03]))

    def test_pandas_advanced(self):
        ampl = self.ampl
        try:
            import pandas as pd
        except ImportError:
            return
        ampl.eval('''
            set someSet := {"First", "Second"};
            param N := 5;
            param x1{someSet, 1..N} default 0;
            param x2{someSet, 1..N} default 0;
            param x3{someSet, 1..N} default 0;
            param x4{someSet, 1..N} default 0;
        ''')

        df = pd.DataFrame([
            [1, 2, 3, 4, 5],
            [6, 7, 8, 9, 0]
        ],
            index=['First', 'Second'],
            columns=[1, 2, 3, 4, 5]
        )
        ampl.param['x1'] = pd.DataFrame.from_dict(
            df.stack().to_dict(),
            orient='index',
            columns=['v']
        )
        ampl.param['x2'] = pd.DataFrame(df.stack())
        ampl.param['x3'] = df.stack().to_dict()
        ampl.param['x4'] = df.stack()
        d1 = ampl.param['x1'].get_values().to_dict()
        d2 = ampl.param['x2'].get_values().to_dict()
        d3 = ampl.param['x3'].get_values().to_dict()
        d4 = ampl.param['x4'].get_values().to_dict()
        self.assertEqual(d1, d2)
        self.assertEqual(d2, d3)
        self.assertEqual(d3, d4)

    def test_pandas_named_columns(self):
        try:
            import pandas as pd
        except ImportError:
            return
        df_unindexed = pd.DataFrame([
            ['Apple', 'Red', 3, 1.29],
            ['Apple', 'Green', 9, 0.99],
            ['Pear', 'Red', 25, 2.59],
            ['Pear', 'Green', 26, 2.79],
            ['Lime', 'Green', 99, 0.39]
        ],
            columns=['Fruit', 'Color', 'Count', 'Price']
        )
        # RangeIndex
        self.assertEqual(DataFrame.from_pandas(
            df_unindexed).get_headers(),
            ('index0', 'Fruit', 'Color', 'Count', 'Price'))

        # MultiIndex
        df_indexed = df_unindexed.set_index(['Fruit', 'Color'])
        self.assertEqual(
            DataFrame.from_pandas(df_indexed,
                                  index_names=['Fruit', 'Color']).get_headers(),
            ('Fruit', 'Color', 'Count', 'Price'))

        # Index without name
        df = pd.DataFrame([
            [1, 2, 3, 4, 5],
            [6, 7, 8, 9, 0]
        ],
            index=['First', 'Second'],
            columns=[1, 2, 3, 4, 5]
        )
        self.assertEqual(DataFrame.from_pandas(
            df.stack()).get_headers(), ('index0', 'index1', '0'))

    def test_numpy(self):
        ampl = self.ampl
        try:
            import numpy as np
        except ImportError:
            return
        ampl.eval('set X;')
        arr = np.array([1, 2, 3])
        ampl.set['X'] = arr
        self.assertEqual(list(ampl.set['X'].members()), [1, 2, 3])
        ampl.eval('param p{1..3};')
        ampl.param['p'] = arr
        self.assertEqual(ampl.param['p'][2], 2)
        self.assertEqual(
            DataFrame.from_numpy(arr).to_list(),
            [1, 2, 3]
        )
        mat = np.array([[1, 2], [3, 4], [5, 6]])
        self.assertEqual(
            DataFrame.from_numpy(mat).to_list(),
            [(1, 2), (3, 4), (5, 6)]
        )
        self.assertEqual(
            DataFrame.from_numpy(mat[:, 0]).to_list(),
            [1, 3, 5]
        )
        self.assertEqual(
            DataFrame.from_numpy(mat[:, 1]).to_list(),
            [2, 4, 6]
        )

    def test_dict(self):
        dic = {'aa': 'bb', 'c': 'a'}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {1: 2}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {1: 2, 3: 4}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {2.0: ('a', 'b'), 3: ('1', '2')}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {(2.0, 'c'): ('a', 'b'), (3, 'a'): ('1', '2')}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        df = DataFrame('x', 'y')
        dic = {1: 12, 2: 23}
        df.set_values(dic)
        self.assertEqual(dic, df.to_dict())
        df = DataFrame('x', ['y', 'z'])
        dic = {1: (12, 2), 2: (23, -1)}
        df.set_values(dic)
        self.assertEqual(dic, df.to_dict())
        df = DataFrame('x', ['y', 'z'])
        df.set_values({1: [1, 2]})
        self.assertEqual({1: (1, 2)}, df.to_dict())

    def test_no_index(self):
        df = DataFrame([], ['x', 'y'])
        x = [1, 2, 3]
        y = [4, 5, 6]
        df.set_column('x', x)
        df.set_column('y', y)
        with self.assertRaises(ValueError):
            df.to_dict()
        pd_df = df.to_pandas()
        self.assertEqual(list(pd_df['x']), x)
        self.assertEqual(list(pd_df['y']), y)

    def test_iter(self):
        df = DataFrame('x', 'y')
        dic = {1: 12, 2: 23}
        df.set_values(dic)
        for row in df:
            self.assertTrue(tuple(row) in [(1, 12), (2, 23)])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import range
# from past.builtins import basestring, unicode


def register_magics(store_name='_ampl_cells', ampl_object=None):
    """
    Register jupyter notebook magics ``%%ampl`` and ``%%ampl_eval``.

    Args:
        store_name: Name of the store where ``%%ampl cells`` will be stored.
        ampl_object: Object used to evaluate ``%%ampl_eval`` cells.
    """
    from IPython.core.magic import (
        Magics, magics_class, cell_magic, line_magic
    )

    @magics_class
    class StoreAMPL(Magics):
        def __init__(self, shell=None,  **kwargs):
            Magics.__init__(self, shell=shell, **kwargs)
            self._store = []
            shell.user_ns[store_name] = self._store

        @cell_magic
        def ampl(self, line, cell):
            """Store the cell in the store"""
            self._store.append(cell)

        @cell_magic
        def ampl_eval(self, line, cell):
            """Evaluate the cell"""
            ampl_object.eval(cell)

        @line_magic
        def get_ampl(self, line):
            """Retrieve the store"""
            return self._store

    get_ipython().register_magics(StoreAMPL)


def multidict(d):
    ncols = min(len(d[k]) for k in d)
    return [list(d.keys())] + [{k: d[k][i] for k in d} for i in range(ncols)]



# -*- coding: utf-8 -*-
"""
AMPLPY
------

AMPL API is an interface that allows developers to access the features of the
AMPL interpreter from within a programming language. All model generation and
solver interaction is handled directly by AMPL, which leads to great stability
and speed; the library just acts as an intermediary, and the added overhead
(in terms of memory and CPU usage) depends mostly on how much data is read
back from AMPL, the size of the model as such is irrelevant. Functions for
directly assigning data to AMPL parameters and sets are provided, which can
be used instead of the normal AMPL data reading procedures.  AMPL API has been
written with usability in mind, and it is easy to access its functionalities
from C++, Java, C#, MATLAB, R and Python.

The AMPL API can function as an add-on to any existing AMPL installation. If
you do not yet have an AMPL installation on the computer where you will be
working with the API, see our
`demo page <http://ampl.com/try-ampl/download-a-free-demo/>`_
or
`trial page <http://ampl.com/try-ampl/request-a-full-trial/>`_
to download a working version that can be installed quickly.

Documentation:
``````````````

* http://amplpy.readthedocs.io
* http://ampl.com/api/nightly/python/

Repositories:
`````````````

* GitHub Repository: https://github.com/ampl/amplpy
* PyPI Repository: https://pypi.python.org/pypi/amplpy
"""
from setuptools import setup, Extension
import platform
import sys
import os

OSTYPE = platform.system()
ARCH = platform.processor()
x64 = platform.architecture()[0] == '64bit'

if ARCH == 'ppc64le':
    LIBRARY = 'ppc64le'
elif ARCH == 'aarch64':
    LIBRARY = 'aarch64'
else:  # 'AMD64', 'x86_64', 'i686', 'i386'
    LIBRARY = 'amd64' if x64 else 'intel32'

if OSTYPE == 'Darwin':
    LIBRARY_EXT = '.dylib'
elif OSTYPE == 'Linux':
    LIBRARY_EXT = '.so'
else:
    LIBRARY_EXT = '.dll'

CPP_BASE = os.path.join('amplpy', 'amplpython', 'cppinterface')
LIBRARY_BASE = os.path.join(CPP_BASE, 'lib')
LIBRARY_DIR = os.path.join(LIBRARY_BASE, LIBRARY)


def ls_dir(base_dir):
    """List files recursively."""
    return [
        os.path.join(dirpath, fname)
        for (dirpath, dirnames, files) in os.walk(base_dir)
        for fname in files
    ]


def package_content():
    all_files = ls_dir('amplpy/')
    if 'sdist' in sys.argv:
        lst = all_files
    else:
        source_only = [
            fpath for fpath in all_files
            if not fpath.startswith(LIBRARY_BASE)
        ]
        library_only = [
            fpath for fpath in all_files
            if fpath.startswith(LIBRARY_DIR)
            if fpath.endswith(LIBRARY_EXT)
        ]
        lst = source_only + library_only
    return [
        fpath.replace('amplpy/', '', 1)
        for fpath in lst
    ]


def compile_args():
    if OSTYPE == 'Windows':
        return ['/TP', '/EHsc']
    elif OSTYPE == 'Linux':
        ignore_warnings = [
            '-Wno-stringop-truncation', 
            '-Wno-catch-value',
            '-Wno-unused-variable',
        ]
        return ['-std=c++11'] + ignore_warnings
    elif OSTYPE == 'Darwin':
        ignore_warnings = [
            '-Wno-unused-variable',
        ]
        return ['-std=c++11', '-mmacosx-version-min=10.9'] + ignore_warnings
    else:
        return []


def link_args():
    rpath = os.path.join(LIBRARY_BASE, LIBRARY)
    if OSTYPE == 'Darwin':
        return ['-Wl,-rpath,@loader_path/' + rpath]
    elif OSTYPE == 'Linux':
        return ['-Wl,-rpath,$ORIGIN/' + rpath]
    else:
        # Return [] instead of [''] for Windows in order to avoid:
        #  cannot open input file '.obj' in build on distutils from Python 3.9
        # https://github.com/pypa/setuptools/issues/2417
        return []


setup(
    name='amplpy',
    version='0.8.0',
    description='Python API for AMPL',
    long_description=__doc__,
    license='BSD-3',
    platforms='any',
    author='Filipe Brandão',
    author_email='fdabrandao@ampl.com',
    url='http://ampl.com/',
    download_url='https://github.com/ampl/amplpy',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: C++',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=['amplpy'],
    ext_modules=[Extension(
        '_amplpython',
        libraries=['ampl'],
        library_dirs=[os.path.join(LIBRARY_BASE, LIBRARY)],
        include_dirs=[os.path.join(CPP_BASE, 'include')],
        extra_compile_args=compile_args(),
        extra_link_args=link_args(),
        sources=[os.path.join(CPP_BASE, 'amplpythonPYTHON_wrap.cxx')],
    )],
    package_data={'': package_content()},
    install_requires=['future >= 0.15.0', 'inflection >= 0.3.1']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from . import TestBase
import unittest
import amplpy


def loadDietModel(ampl):
    ampl.eval('''
        set FOOD ;
        set NUTR ;
        # Parameters
        param cost { FOOD } > 0;
        param f_min { FOOD } >= 0;
        param f_max {j in FOOD } >= f_min [j];
        param n_min { NUTR } >= 0;
        param n_max {i in NUTR } >= n_min [i];
        param amt {NUTR , FOOD } >= 0;
        # Variables
        var Buy {j in FOOD } >= f_min [j], <= f_max [j];
        # Objective
        minimize total_cost : sum {j in FOOD } cost [j] * Buy[j];
        # Contraints
        subject to diet {i in NUTR }:
            n_min [i] <= sum {j in FOOD } amt[i,j] * Buy[j] <= n_max [i];
    ''')
    ampl.eval('''
        data;
        set NUTR := A C B1 B2 NA CAL;
        set FOOD := BEEF CHK FISH HAM MCH MTL SPG TUR ;
        param : cost f_min f_max :=
        BEEF 3.19 2 10
        CHK 2.59 2 10
        FISH 2.29 2 10
        HAM 2.89 2 10
        MCH 1.89 2 10
        MTL 1.99 2 10
        SPG 1.99 2 10
        TUR 2.49 2 10 ;
        param : n_min n_max :=
        A 700 20000
        C 700 20000
        B1 700 20000
        B2 700 20000
        NA 0 50000
        CAL 16000 24000 ;
        param amt (tr):
        A C B1 B2 NA CAL :=
        BEEF 60 20 10 15 938 295
        CHK 8 0 20 20 2180 770
        FISH 8 10 15 10 945 440
        HAM 40 40 35 10 278 430
        MCH 15 35 15 15 1182 315
        MTL 70 30 15 15 896 400
        SPG 25 50 25 15 1329 370
        TUR 60 20 15 10 1397 450 ;
    ''')


class TestEntities(TestBase.TestBase):
    """Test entities."""

    def testVariable(self):
        loadDietModel(self.ampl)
        ampl = self.ampl
        # ampl.solve()
        self.assertEqual(
            ampl.getVariable('Buy').numInstances(),
            ampl.getSet('FOOD').size()
        )
        f_min = ampl.getParameter('f_min')
        f_max = ampl.getParameter('f_max')
        Buy = ampl.getVariable('Buy')
        Buy['BEEF'] = 10
        self.assertEqual(Buy['BEEF'].value(), 10)
        self.assertTrue(isinstance(Buy.getValues(), amplpy.DataFrame))
        self.assertTrue(isinstance(Buy.getValues(['val']), amplpy.DataFrame))
        for index, var in ampl.getVariable('Buy'):
            self.assertTrue(isinstance(var.value(), float))
            var.setValue(f_min[index])
            self.assertEqual(var.value(), f_min[index])
            var.fix()
            self.assertEqual(var.astatus(), 'fix')
            var.unfix()
            self.assertEqual(var.astatus(), 'in')
            var.fix(f_max[index])
            self.assertEqual(var.value(), f_max[index])
            self.assertTrue(isinstance(var.defeqn(), int))
            self.assertTrue(isinstance(var.dual(), float))
            self.assertTrue(isinstance(var.init(), float))
            self.assertTrue(isinstance(var.init0(), float))
            self.assertTrue(isinstance(var.lb(), float))
            self.assertGreaterEqual(var.lb(), f_min[index])
            self.assertTrue(isinstance(var.ub(), float))
            self.assertLessEqual(var.ub(), f_max[index])
            self.assertTrue(isinstance(var.lb0(), float))
            self.assertEqual(var.lb0(), f_min[index])
            self.assertTrue(isinstance(var.ub0(), float))
            self.assertEqual(var.ub0(), f_max[index])
            self.assertTrue(isinstance(var.lb1(), float))
            self.assertGreaterEqual(var.lb1(), f_min[index])
            self.assertTrue(isinstance(var.ub1(), float))
            self.assertLessEqual(var.ub1(), f_max[index])
            self.assertTrue(isinstance(var.lb2(), float))
            self.assertGreaterEqual(var.lb2(), f_min[index])
            self.assertTrue(isinstance(var.ub2(), float))
            self.assertLessEqual(var.ub2(), f_max[index])
            self.assertTrue(isinstance(var.lrc(), float))
            self.assertTrue(isinstance(var.urc(), float))
            self.assertTrue(isinstance(var.lslack(), float))
            self.assertTrue(isinstance(var.uslack(), float))
            self.assertTrue(isinstance(var.rc(), float))
            self.assertTrue(isinstance(var.slack(), float))
            self.assertTrue(isinstance(var.sstatus(), basestring))
            self.assertTrue(isinstance(var.status(), basestring))

    def testConstraint(self):
        loadDietModel(self.ampl)
        ampl = self.ampl
        # ampl.solve()
        self.assertEqual(
            ampl.getConstraint('diet').numInstances(),
            ampl.getSet('NUTR').size()
        )
        for index, con in ampl.getConstraint('diet'):
            self.assertTrue(isinstance(con.isLogical(), bool))
            con.drop()
            con.restore()
            self.assertTrue(isinstance(con.body(), float))
            self.assertTrue(isinstance(con.astatus(), basestring))
            self.assertEqual(con.astatus(), 'in')
            con.drop()
            self.assertEqual(con.astatus(), 'drop')
            con.restore()
            self.assertEqual(con.astatus(), 'in')
            self.assertTrue(isinstance(con.defvar(), int))
            self.assertTrue(isinstance(con.dinit(), float))
            self.assertTrue(isinstance(con.dinit0(), float))
            self.assertTrue(isinstance(con.dual(), float))
            self.assertTrue(isinstance(con.lb(), float))
            self.assertTrue(isinstance(con.ub(), float))
            self.assertTrue(isinstance(con.lbs(), float))
            self.assertTrue(isinstance(con.ubs(), float))
            self.assertTrue(isinstance(con.ldual(), float))
            self.assertTrue(isinstance(con.udual(), float))
            self.assertTrue(isinstance(con.lslack(), float))
            self.assertTrue(isinstance(con.uslack(), float))
            self.assertTrue(isinstance(con.slack(), float))
            self.assertTrue(isinstance(con.status(), basestring))
            self.assertTrue(isinstance(con.sstatus(), basestring))
            con.setDual(0)
            self.assertEqual(con.val(), None)
        ampl.eval('''
            var x;
            var y;
            s.t. xy: x <==> y;
        ''')
        self.assertTrue(ampl.getConstraint('xy').isLogical())
        self.assertTrue(isinstance(ampl.getConstraint('xy').val(), float))

    def testSet(self):
        loadDietModel(self.ampl)
        ampl = self.ampl
        self.assertEqual(ampl.getSet('FOOD').toString(), 'set FOOD;')
        self.assertEqual(str(ampl.getSet('FOOD')), 'set FOOD;')
        self.assertEqual(
            ampl.getSet('NUTR').size(),
            len(ampl.getSet('NUTR').members()),
        )
        self.assertEqual(
            len(ampl.getSet('NUTR').members()),
            len(list(ampl.getSet('NUTR').members()))
        )
        self.assertTrue(ampl.getSet('NUTR').contains('A'))
        self.assertTrue(ampl.getSet('NUTR').contains(('B1',)))
        self.assertFalse(ampl.getSet('NUTR').contains('X'))
        for name, st in ampl.getSets():
            self.assertTrue(isinstance(st.arity(), int))
            self.assertEqual(st.arity(), 1)
            self.assertTrue(isinstance(st.size(), int))
            self.assertEqual(len(st.instances()), 1)
            self.assertEqual(
                len(dict(st.instances())),
                len(list(st.instances()))
            )
            self.assertTrue(isinstance(st.arity(), int))
        ampl.eval('model; set T{1..2}; set T2{1..2};')
        ampl.getSet('T')[1] = [-1, -2]
        self.assertEqual(ampl.getSet('T').get((1,)).getValues().toList(), [-1, -2])
        self.assertEqual(ampl.getSet('T').find((3,)), None)
        self.assertIsInstance(ampl.getSet('T').find(1), amplpy.Set)
        ampl.getSet('T')[1].setValues([1, 2])
        self.assertEqual(ampl.getSet('T')[1].getValues().toList(), [1, 2])
        ampl.getSet('T')[2].setValues(['1', 2])
        self.assertEqual(ampl.getSet('T')[2].getValues().toList(), ['1', 2])
        ampl.getSet('T2')[1].setValues(ampl.getSet('T')[1].getValues())
        self.assertEqual(ampl.getSet('T2')[1].getValues().toList(), [1, 2])
        ampl.getSet('T2')[2].setValues(ampl.getSet('T')[2].getValues())
        self.assertEqual(ampl.getSet('T2')[2].getValues().toList(), ['1', 2])
        ampl.eval('set T3 dimen 2; set T4 dimen 2;')
        ampl.getSet('T3').setValues([(1, 2)])
        self.assertEqual(ampl.getSet('T3').getValues().toList(), [(1, 2)])
        ampl.getSet('T3').setValues(set([('b', 1), ('a', 2)]))
        self.assertEqual(sorted(ampl.getSet('T3').getValues().toList()), [('a', 2), ('b', 1)])
        ampl.eval('set T5;')
        ampl.set['T5'] = set(['a', 'b', 'c'])
        self.assertEqual(sorted(ampl.getSet('T5').getValues().toList()), ['a', 'b', 'c'])

        try:
            import numpy as np
        except ImportError:
            pass
        else:
            ampl.getSet('T')[1].setValues(np.array([1, 2]))
            self.assertEqual(ampl.getSet('T')[1].size(), 2)
            ampl.getSet('T3').setValues(np.array([[1, 2], [3, 4]]))
            self.assertEqual(ampl.getSet('T3').size(), 2)

    def testParameter(self):
        loadDietModel(self.ampl)
        ampl = self.ampl
        self.assertEqual(
            ampl.getSet('FOOD').size(),
            ampl.getParameter('cost').numInstances()
        )
        ampl.eval('''
            param a;
            param b default a;
            param c symbolic;
            param d{1..2} symbolic;
            param d2{1..2} symbolic;
        ''')
        self.assertFalse(ampl.getParameter('a').hasDefault())
        self.assertTrue(ampl.getParameter('b').hasDefault())
        self.assertFalse(ampl.getParameter('a').isSymbolic())
        self.assertFalse(ampl.getParameter('b').isSymbolic())
        self.assertTrue(ampl.getParameter('c').isSymbolic())
        cost = ampl.getParameter('cost')
        self.assertEqual(cost.indexarity(), 1)
        self.assertEqual(cost.getIndexingSets(), ['FOOD'])
        self.assertEqual(cost.xref(), ['total_cost'])
        for i, food in enumerate(ampl.getSet('FOOD').members()):
            cost[food] = i+1
            self.assertEqual(cost[food], i+1)
            cost.set(food, i+2)
            self.assertEqual(cost[food], i+2)
        a = ampl.getParameter('a')
        a.set(10)
        self.assertEqual(a.value(), 10)
        c = ampl.getParameter('c')
        b = ampl.getParameter('c')
        c.set('a')
        self.assertEqual(c.value(), 'a')
        self.assertEqual(b.value(), 'a')
        self.assertEqual(b.indexarity(), 0)
        self.assertEqual(a.isScalar(), True)
        self.assertEqual(b.isScalar(), True)
        self.assertEqual(c.isScalar(), True)
        d = ampl.getParameter('d')
        self.assertEqual(d.isScalar(), False)
        d.set(1, 'a')
        self.assertEqual(d[1], 'a')
        d[1] = 'aa'
        self.assertEqual(d[1], 'aa')
        d.set(2, 'b')
        self.assertEqual(d[2], 'b')
        d.setValues({1: 'x', 2: 'y'})
        self.assertEqual(d[1], 'x')
        self.assertEqual(d[2], 'y')
        d.setValues(['xx', 'yy'])
        self.assertEqual(d[1], 'xx')
        self.assertEqual(d[2], 'yy')
        d2 = ampl.getParameter('d2')
        d2.setValues(d.getValues())
        self.assertEqual(d2[1], 'xx')
        self.assertEqual(d2[2], 'yy')
        self.assertEqual(cost.isScalar(), False)
        ampl.eval('param cost2{FOOD};')
        self.assertTrue(isinstance(cost.getValues(), amplpy.DataFrame))
        cost2 = ampl.getParameter('cost2')
        cost2.setValues(cost.getValues())
        for food in ampl.getSet('FOOD').members():
            self.assertEqual(cost2[food], cost[food])

        try:
            import numpy as np
        except ImportError:
            pass
        else:
            ampl.eval('param lst{1..3};')
            ampl.getParameter('lst').setValues(np.array([1, 2, 3]))

    def testPrecision(self):
        PI = 3.1415926535897932384626433832795028841971
        ampl = self.ampl
        ampl.eval('param x;')
        ampl.getParameter('x').set(PI)
        ampl.eval("display x;")  # invalidate cache
        self.assertEqual(ampl.param['x'].value(), PI)

    def testInfinity(self):
        inf = float('inf')
        ampl = self.ampl
        ampl.eval('param x;')
        ampl.getParameter('x').set(inf)
        self.assertEqual(ampl.getValue('x'), inf)
        ampl.eval('param y{1..3};')
        ampl.getParameter('y').setValues([inf]*3)
        for i in range(3):
            self.assertEqual(ampl.getValue('y[{}]'.format(i+1)), inf)

    def testObjective(self):
        loadDietModel(self.ampl)
        ampl = self.ampl
        obj = ampl.getObjective('total_cost')
        self.assertEqual(ampl.getCurrentObjective().name(), 'total_cost')
        self.assertEqual(
            len(dict(obj)),
            obj.numInstances()
        )
        self.assertTrue(isinstance(obj.value(), float))
        self.assertTrue(isinstance(obj.astatus(), basestring))
        self.assertTrue(isinstance(obj.sstatus(), basestring))
        self.assertTrue(isinstance(obj.exitcode(), int))
        self.assertTrue(isinstance(obj.message(), basestring))
        self.assertTrue(isinstance(obj.result(), basestring))
        obj.drop()
        self.assertEqual(obj.astatus(), 'drop')
        obj.restore()
        self.assertEqual(obj.astatus(), 'in')
        self.assertTrue(obj.minimization())
        ampl.eval('''
            maximize A: 1;
            minimize B: 1;
            maximize C: 1;
        ''')
        self.assertEqual(ampl.getCurrentObjective(), None)
        ampl.eval('objective A;')
        self.assertEqual(ampl.getCurrentObjective().name(), 'A')
        self.assertFalse(ampl.getCurrentObjective().minimization())
        ampl.eval('objective B;')
        self.assertEqual(ampl.getCurrentObjective().name(), 'B')
        self.assertTrue(ampl.getCurrentObjective().minimization())
        ampl.eval('objective C;')
        self.assertEqual(ampl.getCurrentObjective().name(), 'C')
        self.assertFalse(ampl.getCurrentObjective().minimization())
        ampl.eval('objective A;')
        self.assertEqual(ampl.getCurrentObjective().name(), 'A')
        self.assertFalse(ampl.getCurrentObjective().minimization())

    def testSetValues(self):
        ampl = self.ampl
        ampl.eval('var x{1..3};')
        d = {1: 2, 2: 5, 3: 9}
        ampl.var['x'].setValues(d)
        self.assertEqual(d, ampl.var['x'].getValues().toDict())

        d = {1: 11, 2: 55, 3: 99}
        ampl.var['x'] = d
        self.assertEqual(d, ampl.var['x'].getValues().toDict())


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division

# from builtins import map, range, object, zip, sorted
from builtins import map
from numbers import Real
from past.builtins import basestring

from .entity import Entity
from .dataframe import DataFrame
try:
    import numpy as np
except ImportError:
    np = None


class Parameter(Entity):
    """
    Represents an AMPL parameter. The values can be float or string (in case of
    symbolic parameters).

    Data can be assigned to the set using the methods
    :func:`~amplpy.Parameter.set` and
    :func:`~amplpy.Parameter.set_values` / :func:`~amplpy.Parameter.setValues`
    or using
    :func:`~amplpy.AMPL.set_data` / :func:`~amplpy.AMPL.setData`
    and an object of class :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        Entity.__init__(
            self,
            _impl,
            lambda it: it
        )

    def __setitem__(self, index, value):
        self.set(index, value)

    def is_symbolic(self):
        """
        Returns True if the parameter is declared as symbolic (can store both
        numerical and string values).
        """
        return self._impl.isSymbolic()

    def has_default(self):
        """
        Check if the parameter has a default initial value. In case of the
        following AMPL code:

        .. code-block:: ampl

            param a;
            param b default a;

        the function will return true for parameter ``b``.

        Returns:
            True if the parameter has a default initial value. Please note
            that if the parameter has a default expression which refers to
            another parameter which value is not defined, this will return
            True.
        """
        return self._impl.hasDefault()

    def value(self):
        """
        Get the value of this parameter. Valid only for non-indexed parameters.
        """
        return self.get()

    def set(self, *args):
        """
        Set the value of a single instance of this parameter.

        Args:
            args: value if the parameter is scalar, index and value
            otherwise.

        Raises:
            RuntimeError: If the entity has been deleted in the underlying
            AMPL.

            TypeError: If the parameter is not scalar and the index is not
            provided.
        """
        assert len(args) in (1, 2)
        if len(args) == 1:
            value = args[0]
            self._impl.set(value)
        else:
            index, value = args
            if isinstance(value, Real):
                self._impl.setTplDbl(index, value)
            elif isinstance(value, basestring):
                self._impl.setTplStr(index, value)
            else:
                raise TypeError

    def set_values(self, values):
        """
        Assign the values (string or float) to the parameter instances with the
        specified indices, equivalent to the AMPL code:

        .. code-block:: ampl

            let {i in indices} par[i] := values[i];

        Args:
            values: list, dictionary or :class:`~amplpy.DataFrame` with the
            indices and the values to be set.

        Raises:
            TypeError: If called on a scalar parameter.
        """
        if isinstance(values, dict):
            if not values:
                return
            if self._impl.setValuesPyDict(values) != 0:
                raise TypeError
            # indices, values = list(values.keys()), values.values()
            # if any(isinstance(value, basestring) for value in values):
            #     values = list(map(str, values))
            #     self._impl.setValuesTaStr(indices, values, len(values))
            # elif all(isinstance(value, Real) for value in values):
            #     values = list(map(float, values))
            #     self._impl.setValuesTaDbl(indices, values, len(values))
            # else:
            #     raise TypeError
        elif isinstance(values, (list, tuple)):
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.setValuesStr(values, len(values))
            elif all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.setValuesDbl(values, len(values))
            else:
                raise TypeError
        else:
            if np is not None and isinstance(values, np.ndarray):
                self.set_values(DataFrame.from_numpy(values).to_list())
                return
            Entity.set_values(self, values)

    # Aliases
    hasDefault = has_default
    isSymbolic = is_symbolic
    setValues = set_values
